/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "usbscanwork.h"

GvfsMountManager *GvfsMountManager::m_gvfsMountManager = nullptr;

GvfsMountManager::GvfsMountManager(QObject *parent)
    :QObject(parent)
{
    // usb dissection
    GVolumeMonitor* monitor = g_volume_monitor_get();
    g_signal_connect_data ((monitor), ("mount-added"), GCallback(&GvfsMountManager::onMountAdded), gpointer("mount-added"), nullptr, GConnectFlags::G_CONNECT_AFTER);
}

GvfsMountManager *GvfsMountManager::instance()
{
    if (m_gvfsMountManager == nullptr)
        m_gvfsMountManager = new GvfsMountManager;

    return m_gvfsMountManager;
}
// 接受到卷监视器的挂载信号时的槽函数
void GvfsMountManager::onMountAdded(GVolumeMonitor* monitor, GMount* mount, gpointer data)
{
    Q_UNUSED(monitor);
    char *ev = (char*)data;
    g_print("[handle_mount_changed] event: %s\n", ev);
    // 如果是挂载移除操作，则不处理
    if (g_strcmp0(ev, "mount-removed") == 0) {
        return;
    }

    // 如果是不可移除设备，则不处理
    if(!g_mount_can_eject(mount)) {
        qDebug() << "device can not eject";
        return;
    }
    // 获取分区挂载路径，并发送挂载信号
    GFile *root = g_mount_get_root(mount);
    char *path = g_file_get_path(root);
    g_print("mount path: %s\n", path);
    Q_EMIT GvfsMountManager::instance()->mountAdded(path);
}

UsbScanWork::UsbScanWork(QObject *parent)
    : QObject(parent)
    , m_AntivirusServer(nullptr)
    , m_gsetting(nullptr)
    , m_scanStatus(ScanState::Continue)
    , m_isbUsbDissection(false)
    , m_sUsbScanPath("")
{
    // 病毒查杀服务dbus接口
    m_AntivirusServer = new AntivirusServer("com.deepin.defender.antiav",
                                            "/com/deepin/defender/antiav",
                                            QDBusConnection::systemBus(), this);
    // 安全中心gsetting
    m_gsetting = new QGSettings("com.deepin.dde.deepin-defender", QByteArray(), this);

    // 连接扫描结果信号
    connect(m_AntivirusServer, &AntivirusServer::onDissectionResult, this, &UsbScanWork::onRecDissectionResult);
    connect(m_AntivirusServer, &AntivirusServer::signal_doDissectionFinished, this, &UsbScanWork::onScanningFinished);

    // 连接挂载添加信号
    connect(GvfsMountManager::instance(), &GvfsMountManager::mountAdded, this, &UsbScanWork::onMountAdded);
}

UsbScanWork::~UsbScanWork()
{

}

void UsbScanWork::doDissection(bool isbAll, QString sPaths)
{
    m_AntivirusServer->doDissection(isbAll, sPaths);
    requestSetDissectionStatus(ScanState::Continue);
}

void UsbScanWork::requestSetDissectionStatus(ScanState status)
{
    int iStatus = 1;
    // 转换扫描状态为int型
    switch (status) {
    case ScanState::Pause:
        iStatus = 0;
        break;
    case ScanState::Continue:
        iStatus = 1;
        break;
    case ScanState::Stop:
        iStatus = 2;
        break;
    }
    m_scanStatus = status;
    m_AntivirusServer->setDissection(iStatus);
}

bool UsbScanWork::isScanning()
{
    // 如果服务正在扫描 或 处于扫描界面，则返回true
    return m_AntivirusServer->isScanning();
}

void UsbScanWork::onRecDissectionResult(QString sPath, double dProgress, int iType, QString sDetailInfo)
{
    Q_UNUSED(sDetailInfo);
    qDebug() << "sPath" << sPath << ", dProgress" << dProgress;

    ScanResultType type = ScanResultType(iType);
    // 如果当前为USB查杀，并且检测到异常文件，则显示USB危险系统通知
    if (ScanState::Continue == m_scanStatus &&
        m_isbUsbDissection &&
        type != ScanResultType::Normal
        ) {
        showUsbHaveDangersSysNotification();
        requestSetDissectionStatus(ScanState::Stop);
        setIsbUsbDissection(false);
    }
}

void UsbScanWork::onScanningFinished()
{
    // 扫描完成时,如果当前为USB查杀，设置 usb查杀标志为false
    if (ScanState::Continue == m_scanStatus &&
        m_isbUsbDissection
        ) {
        qDebug() << "usb storage scanning finished !";
        requestSetDissectionStatus(ScanState::Stop);
        setIsbUsbDissection(false);
    }
}

void UsbScanWork::onMountAdded(QString sMountPoint)
{
    bool isbAutoScanUsb = m_gsetting->get("auto-scan-usb").toBool();
    if (!isbAutoScanUsb)
        return;

    if (isScanning()) {
        setIsbUsbDissection(false);
        showUsbScanWhenBusySysNotification();
    } else {
        setIsbUsbDissection(true);
        showUsbScanningSysNotification();
        doDissection(false, sMountPoint);
        m_sUsbScanPath = sMountPoint;
    }
}

void UsbScanWork::setIsbUsbDissection(bool isbUsb)
{
    m_isbUsbDissection = isbUsb;
}

void UsbScanWork::showUsbScanningSysNotification()
{
    qDebug() << "showUsbScanningSysNotification";
    QDBusMessage m = QDBusMessage::createMethodCall("com.deepin.dde.Notification"
                                                    , "/com/deepin/dde/Notification"
                                                    , "com.deepin.dde.Notification"
                                                    , "Notify");
    QStringList m_action;
    m_action << "_open1" << tr("Settings"); //添加按钮
    QVariantMap m_inform;//按钮的点击操作
    QString openAntivirusPageCmd = "qdbus,com.deepin.defender.hmiscreen,"
                                   "/com/deepin/defender/hmiscreen,"
                                   "com.deepin.defender.hmiscreen.showSettingDialog,"
                                   "base";

    qDebug() << openAntivirusPageCmd;
    m_inform.insert(QString("x-deepin-action-_open1"), openAntivirusPageCmd);
    m << QString("deepin-defender")
      << uint(0)
      << QString("deepin-defender")
      << QString("") << tr("Scanning the USB device...")
      << m_action
      << m_inform
      << int(3000);
    QDBusMessage respone = QDBusConnection::sessionBus().call(m);
}

void UsbScanWork::showUsbScanWhenBusySysNotification()
{
    qDebug() << "showUsbScanWhenBusySysNotification";
    QDBusMessage m = QDBusMessage::createMethodCall("com.deepin.dde.Notification"
                                                    , "/com/deepin/dde/Notification"
                                                    , "com.deepin.dde.Notification"
                                                    , "Notify");
    QStringList m_action;
    m_action << "_open1" << tr("Settings"); //添加按钮
    QVariantMap m_inform;//按钮的点击操作
    QString openAntivirusPageCmd = "qdbus,com.deepin.defender.hmiscreen,"
                                   "/com/deepin/defender/hmiscreen,"
                                   "com.deepin.defender.hmiscreen.showSettingDialog,"
                                   "base";

    qDebug() << openAntivirusPageCmd;
    m_inform.insert(QString("x-deepin-action-_open1"), openAntivirusPageCmd);
    m << QString("deepin-defender")
      << uint(0)
      << QString("deepin-defender")
      << QString("") << tr("Scan the USB device manually after current scanning completed")
      << m_action
      << m_inform
      << int(3000);
    QDBusMessage respone = QDBusConnection::sessionBus().call(m);
}

void UsbScanWork::showUsbHaveDangersSysNotification()
{
    qDebug() << "showUsbHaveDangersSysNotification" << m_sUsbScanPath;
    QDBusMessage m = QDBusMessage::createMethodCall("com.deepin.dde.Notification"
                                                    , "/com/deepin/dde/Notification"
                                                    , "com.deepin.dde.Notification"
                                                    , "Notify");
    QStringList m_action;
    m_action << "_open1" << tr("Scan Now"); //添加按钮
    QVariantMap m_inform;//按钮的点击操作
    QString openAntivirusPageCmd = "qdbus,com.deepin.defender.hmiscreen,"
                                   "/com/deepin/defender/hmiscreen,"
                                   "com.deepin.defender.hmiscreen.showScanByPath,"
                                   + m_sUsbScanPath;

    qDebug() << openAntivirusPageCmd;
    m_inform.insert(QString("x-deepin-action-_open1"), openAntivirusPageCmd);
    m << QString("deepin-defender")
      << uint(0)
      << QString("deepin-defender")
      << QString("") << tr("Your USB device is unsafe")
      << m_action
      << m_inform
      << int(5000);
    QDBusMessage respone = QDBusConnection::sessionBus().call(m);
}
