/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "startupmonitor.h"

#include <QDir>
#include <QFileInfo>
#include <QDebug>

#include <pwd.h>
#include <unistd.h>

StartupMonitor *StartupMonitor::m_pInstance = nullptr;
StartupMonitor::StartupMonitor(QObject *parent) :
    QObject(parent)
{
}

// 监控文件或目录
void StartupMonitor::addWatchPath(QString path)
{
    // 取HOME路径
    if (path.contains("~"))
        path = path.replace(0, 1, getenv("HOME"));

    // 初始化监控器
    qDebug() << QString("Add to watch: %1").arg(path);
    if (m_pInstance == nullptr) {
        m_pInstance = new StartupMonitor(this);
        m_pInstance->m_pWatcherDir = new QFileSystemWatcher();
        m_pInstance->m_pWatcherFile = new QFileSystemWatcher();

        // 连接QFileSystemWatcher的directoryChanged和fileChanged信号到相应的槽
        connect(m_pInstance->m_pWatcherDir, SIGNAL(directoryChanged(QString)), m_pInstance, SLOT(directoryUpdated(QString)));
        connect(m_pInstance->m_pWatcherFile, SIGNAL(fileChanged(QString)), m_pInstance, SLOT(fileUpdated(QString)));
    }

    // 添加文件路径监控
    QDir fileDir(path);
    QList<QFileInfo> fileInfo = fileDir.entryInfoList();
    for (int i = 0; i < fileInfo.count(); ++i) {
        QString sFilePath = QDir::cleanPath(fileInfo.at(i).filePath());
        m_pInstance->m_pWatcherFile->addPath(sFilePath);
    }

    // 添加目录路径监控
    bool bAdd = m_pInstance->m_pWatcherDir->addPath(path);
    qDebug() << QString("Add to watch: %1").arg(bAdd);

    // 如果添加路径是一个目录，保存当前内容列表
    QFileInfo file(path);
    if (file.isDir()) {
        const QDir dirw(path);
        m_pInstance->m_currentContentsMap[path] = dirw.entryList(QDir::NoDotAndDotDot | QDir::AllDirs | QDir::Files, QDir::DirsFirst);
    }
}

// 文件修改时调用
void StartupMonitor::fileUpdated(const QString &filePath)
{
    QFileInfo file(filePath);
    QString strPath = file.absolutePath();
    QString strName = file.fileName();
    m_sFileName = strName;

    qDebug() << QString("The file %1 at path %2 is updated").arg(strName).arg(strPath);

    QFile fileInfo(strPath + "/" + strName);
    if (fileInfo.open(QIODevice::ReadOnly | QIODevice::Text)) {
        QTextStream in(&fileInfo);
        QString sAllInfo = in.readAll();

        if (sAllInfo.contains("Hidden=true")) {
            doDataChange(false);
        } else if (!sAllInfo.contains("Hidden")) {
            QTimer::singleShot(200, this, SLOT(delayAction()));
        } else {
            doDataChange(true);
        }
    }
}

// 只要任何监控的目录更新（添加、删除、重命名），就会调用。
void StartupMonitor::directoryUpdated(const QString &dirPath)
{
    qDebug() << QString("Directory updated: %1").arg(dirPath);

    // 比较最新的内容和保存的内容找出区别(变化)
    QStringList currEntryList = m_currentContentsMap[dirPath];
    const QDir dir(dirPath);

    QStringList newEntryList = dir.entryList(QDir::NoDotAndDotDot  | QDir::AllDirs | QDir::Files, QDir::DirsFirst);

    QSet<QString> newDirSet = QSet<QString>::fromList(newEntryList);
    QSet<QString> currentDirSet = QSet<QString>::fromList(currEntryList);

    // 添加了文件
    QSet<QString> newFiles = newDirSet - currentDirSet;
    QStringList newFile = newFiles.toList();

    // 文件已被移除
    QSet<QString> deletedFiles = currentDirSet - newDirSet;
    QStringList deleteFile = deletedFiles.toList();

    // 更新当前设置
    m_currentContentsMap[dirPath] = newEntryList;

    if (!newFile.isEmpty() && !deleteFile.isEmpty()) {
        // 文件/目录重命名
        if ((newFile.count() == 1) && (deleteFile.count() == 1)) {

            m_pInstance->m_pWatcherFile->removePath(dirPath + "/" + deleteFile.first());
            m_pInstance->m_pWatcherFile->addPath(dirPath + "/" + newFile.first());
            m_sFilePath = dirPath + "/" + newFile.first();
            m_sFileName = newFile.first();
            QTimer::singleShot(200, this, SLOT(delayAction()));
            qDebug() << QString("File Renamed from %1 to %2").arg(deleteFile.first()).arg(newFile.first());
        }
    } else {
        // 添加新文件/目录至Dir
        if (!newFile.isEmpty()) {
            qDebug() << "New Files/Dirs added: " << newFile;

            foreach (QString file, newFile) {
                // 处理操作每个新文件....
                m_pInstance->m_pWatcherFile->addPath(dirPath + "/" + file);
                m_sFilePath = dirPath + "/" + file;
                m_sFileName = file;
                QTimer::singleShot(200, this, SLOT(delayAction()));
            }
        }

        // 从Dir中删除文件/目录
        if (!deleteFile.isEmpty()) {
            qDebug() << "Files/Dirs deleted: " << deleteFile;

            foreach (QString file, deleteFile) {
                // 处理操作每个被删除的文件....
                m_pInstance->m_pWatcherFile->removePath(dirPath + "/" + file);
                doDataChange(false);
            }
        }
    }
}
// 延迟调用响应函数
void StartupMonitor::delayAction()
{
    checkActionSource(m_sFilePath, m_sFileName);
}

// 检测是否是第三方开启 开机自启动
void StartupMonitor::checkActionSource(QString sFilePath, QString sAppsName)
{
    QFile fileInfo(sFilePath);
    if (fileInfo.open(QIODevice::ReadOnly | QIODevice::Text)) {
        QTextStream in(&fileInfo);
        QString sAllInfo = in.readAll();

        doDataChange(true);
        if (!sAllInfo.contains("Hidden")) {
            qDebug() << "Third-party: appsinfo:  " << sAllInfo;

            if ((m_sLastFileName == sAppsName) && m_dialog != nullptr) {
                return;
            }

            // 弹框允许弹出时 复制APP名称到最新保存值
            m_sLastFileName = sAppsName;

            // 开机自启拦截弹框
            m_dialog = new StartupMonitorDialog;
            connect(m_pInstance->m_dialog, SIGNAL(monitorChange()), m_pInstance, SLOT(doDataRefresh()));
            connect(m_pInstance->m_dialog, SIGNAL(onDialogClose()), m_pInstance, SLOT(doDialogClose()));

            m_dialog->setAppsName(sAppsName);
            m_dialog->setAppsPath(sFilePath);
            m_dialog->setAttribute(Qt::WA_DeleteOnClose);
            m_dialog->setVisible(true);
        } else if (sAllInfo.contains("Hidden=true")) {
            doDataChange(false);
        }
    }
}

void StartupMonitor::doDataChange(bool isStartup)
{
    qDebug() << "StartupMonitor--filename:  " << m_sFileName;
    QStringList sList = m_sFileName.split(".");
    QString sID;
    if (sList.count() > 0)
        sID = sList.at(0);
    if (sList.count() > 2) {
        for (int i = 1; i < sList.count() - 1; ++i) {
            sID  += "." + sList.at(i);
        }
    }
    qDebug() << "StartupMonitor--id:  " << sID;

    Q_EMIT m_pInstance->doDataChage(isStartup, sID);
}

StartupMonitor *StartupMonitor::getInstance()
{
    return m_pInstance;
}

// 表格数据刷新
void StartupMonitor::doDataRefresh()
{
    doDataChange(false);
}

// 弹框关闭槽
void StartupMonitor::doDialogClose()
{
    m_dialog = nullptr;
}
