#include "usbmanagemonitor.h"
#include "libudev.h"
#include "stdio.h"    // printf(3)
#include "stdlib.h"   // exit(3)
#include "unistd.h"   // fork(3), chdir(3), sysconf(3)
#include "signal.h"   // signal(3)
#include "sys/stat.h" // umask(3)
#include "syslog.h"   // syslog(3), openlog(3), closelog(3)
#include "string.h"   // strncmp
#include "aio.h"      // async I/O
#include "sys/types.h" // aiocb datatype
#include "errno.h"     // EINPROGRESS
#include "fcntl.h"    // open() file descriptor
#include "string.h"   // strsep(3)
#include "proc/readproc.h"

#include <QDebug>
#include <QCoreApplication>
#include <QFile>
#include <QProcess>
#include <QThread>
#include <QSettings>
#include <QDir>
#include <QDateTime>
#include <fstream>

#define SUBSYSTEM                               "usb"
// 全部允许/允许白名单/全部禁止
#define USB_LIMIT_ALL                           "0"
#define USB_LIMIT_PARTY                         "1"
#define USB_LIMIT_ZERO                          "2"
// 配置目录
#define CONFIG_DIR                              "/usr/bin/config/deepin-defender/"
// 配置文件名
#define CONFIG_FILE_NAME                        "usbconnectionconfig.ini"
// 连接记录备份文件名
#define USB_CONNECTION_LOG_BAK_FILE_NAME        "usbconnection.log.bak"
// 连接记录文件名
#define USB_CONNECTION_LOG_FILE_NAME            "usbconnection.log"
// 白名单备份文件名
#define USB_WHITELIST_BAK_FILE_NAME             "usbconnectionwhitelist.list.bak"
// 白名单文件名
#define USB_WHITELIST_FILE_NAME                 "usbconnectionwhitelist.list"

//文件解加密钥匙
const int CryptionBufferSize = 8;
const char CryptionBuffer[CryptionBufferSize + 1] = "defender";

UsbManageMonitor::UsbManageMonitor(QObject *parent)
    : QObject(parent)
    , m_udev(nullptr)
    , m_config(nullptr)
    , m_sLimitModel("0")
    , m_whiteList()
    , m_isbManagedDiskStorage(false)
    , m_isbManagedMobileStorage(false)
{
    loadConfig();
    readConnectionWhiteLst();
    readConnectionRecord();
    //清除超过30天记录
    cleanUsbConnectionLog();

    m_status = 1;
    m_disConnectFlag = 1;

    m_usbDeviceInfo.sSysPath = "*";

    preManage();

    connect(this, &UsbManageMonitor::sendConnectionRecord, [ = ](QString sInfos) {
        qDebug() << "sendConnectionRecord:" << sInfos;
    });
}

UsbManageMonitor::~UsbManageMonitor()
{
    m_config->deleteLater();

    this->quit();
    this->deleteLater();

    this->thread()->quit();
    connect(this->thread(), &QThread::finished, this->thread(), &QThread::deleteLater);
}

void UsbManageMonitor::init()
{

}

void UsbManageMonitor::run()
{
    // 首先调用udev_new，创建一个udev library context。
    // udev library context采用引用记数机制，创建的context默认引用记数为1，
    // 使用udev_ref和udev_unref增加或减少引用记数，如果引用记数为0，则释放内部资源
    m_udev = udev_new();
    if (!m_udev) {
        qDebug() << "Can't create udev.";
    } else {
        qDebug() << "Finished create udev.";
    }

    // 监控设备插拔 udev的设备插拔基于netlink实现
    monitorDevices(m_udev);

    qDebug() << "finish udev!";
}

void UsbManageMonitor::quit()
{
    m_status = 0;
}

void UsbManageMonitor::loadConfig()
{
    //通过daemonservice进程获取用户目录路径
//    QString sMainWindowUserName = getProcUserName("deepin-defender-daemonservice");
//    if (sMainWindowUserName.isEmpty()) {
//        qDebug() << "deepin-defender-daemonservice have not run, load config error!";
//    }

//    m_configPath = QString("/home/").append(sMainWindowUserName);
//    m_configPath.append("/deepin-defender/");
    m_configPath = "/usr/bin/config/deepin-defender/";
    qDebug() << "m_configPath:" << m_configPath;

    //读取限制模式配置
    QString sSettingsPath = m_configPath + "usbconnectionconfig.ini";
    m_config = new QSettings(sSettingsPath, QSettings::IniFormat);
    m_config->setParent(this);
    QString value = m_config->value("SaveRecordSwitch").toString();
    if (value.isEmpty()) {
        m_config->setValue("SaveRecordSwitch", "false");
        value = "false";
    }
    if ("true" == value) {
        m_isbSaveRecord = true;
    } else {
        m_isbSaveRecord = false;
    }

    value = m_config->value("LimitaionModel").toString();
    if (value.isEmpty()) {
        m_config->setValue("LimitaionModel", "0");
        value = "0";
    }
    m_sLimitModel = value;
    m_config->sync();

    //修改用户目录权限为777
    QProcess proc;
    proc.start(QString("sudo chmod 777 %1").arg(sSettingsPath));
    proc.waitForFinished();
    proc.deleteLater();
}

void UsbManageMonitor::readConnectionRecord()
{
    //name--time--isbBlocked--serial--vendorID--productID
    QDir dir;
    //目录不存在，就创建
    if (!dir.exists(m_configPath)) {
        dir.mkpath(m_configPath);
    }

    // 连接记录备份文件的路径
    QString m_connectionLogBackupPath = m_configPath + QString(USB_CONNECTION_LOG_BAK_FILE_NAME);
    // 源连接记录文件路径
    QString m_connectionLogPath = m_configPath + QString(USB_CONNECTION_LOG_FILE_NAME);
    // 优先读取备份文件数据
    QFile f(m_connectionLogBackupPath);
    if (!f.exists()) {
        f.setFileName(m_connectionLogPath);
    }

    if (!f.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "Open usbconnection.log failed." << endl;
        return;
    }

    m_usbConnectionInfoLst.clear();
    QTextStream txtOutout(&f);
    //name--time--isbBlocked--serial--vendorID--productID--sParentSize--sSize
    QString sRecords = txtOutout.readAll();

    QTextStream txtOutput(&sRecords);
    QString lineStr;
    while (!txtOutput.atEnd()) {
        lineStr = txtOutput.readLine();
        qDebug() << lineStr << endl;
        QStringList infoLst = lineStr.split(",");
        if (8 != infoLst.size()) continue;
        //usb连接日志信息结构
        UsbConnectionInfo info;
        info.sName = infoLst.at(0);
        info.sDatetime = infoLst.at(1);
        info.sStatus = infoLst.at(2);
        info.sSerial = infoLst.at(3);
        info.sVendorID = infoLst.at(4);
        info.sProductID = infoLst.at(5);
        info.sParentSize = infoLst.at(6);
        info.sSize = infoLst.at(7);

        m_usbConnectionInfoLst.append(info);
    }

    // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
    int fd = f.handle();
    fsync(fd);
    f.close();
}

void UsbManageMonitor::readConnectionWhiteLst()
{
    //name--time--isbBlocked--serial--vendorID--productID
    QDir dir;
    //目录不存在，就创建
    if (!dir.exists(m_configPath)) {
        dir.mkpath(m_configPath);
    }

    // 备份文件的文件名
    QString m_whiteListBackupPath = m_configPath + QString(USB_WHITELIST_BAK_FILE_NAME);
    // 源白名单文件路径
    QString m_whiteListPath = m_configPath + QString(USB_WHITELIST_FILE_NAME);
    // 优先读取备份文件数据
    QFile f(m_whiteListBackupPath);
    if (!f.exists()) {
        f.setFileName(m_whiteListPath);
    }

    if (!f.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "Open usbconnectionwhitelist.list failed." << endl;
        return;
    }

    m_whiteList.clear();
    //先解密
    //name--time--isbBlocked--serial--vendorID--productID--sParentSize--sSize
    f.seek(0);
    QByteArray ba = f.readAll();
    QString whiteListAllStr = decryptBinary(ba);
    qDebug() << whiteListAllStr;

    QStringList whiteListStrLine = whiteListAllStr.split("\n");
    for (int i =0; i < whiteListStrLine.size(); i++) {
        QStringList infoLst = whiteListStrLine.at(i).split(",");
        if (8 != infoLst.size()) continue;
        //usb连接信息结构
        UsbConnectionInfo info;
        info.sName = infoLst.at(0);
        info.sDatetime = infoLst.at(1);
        info.sStatus = infoLst.at(2);
        info.sSerial = infoLst.at(3);
        info.sVendorID = infoLst.at(4);
        info.sProductID = infoLst.at(5);
        info.sParentSize = infoLst.at(6);
        info.sSize = infoLst.at(7);

        m_whiteList.append(info);
    }

    // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
    int fd = f.handle();
    fsync(fd);
    f.close();
}

void UsbManageMonitor::disconnectAllDevices()
{
    QProcess *proc = new QProcess;
    QString cmd = "lsblk -nro TYPE,PATH,MOUNTPOINT,HOTPLUG";
    proc->start(cmd);
    proc->waitForFinished();

    QString allInfoStr = proc->readAllStandardOutput().trimmed();
    qDebug() << "allInfoStr" << allInfoStr;

    QList<BlkInfo> blkInfoList = {};
    QStringList infoFirstCut = allInfoStr.split("\n");
    //添加结束辅助分区信息，用于管控最后一个分区
    infoFirstCut.append("disk 0 0 0");
    QString parentDevPath = "";
    bool isbHaveSysMountPoint = false;
    QString sCanRemove = "0";
    for (int i = 0; i < infoFirstCut.count(); i++) {
        QStringList infoSecondCut = infoFirstCut.at(i).split(" ");
        // 获取设备类型-"disk" "part"
        QString type = infoSecondCut.at(0);
        // 如果是硬盘则获取主设备dev节点和是否可移除属性
        if ("disk" == type) {
            if (!parentDevPath.isEmpty() && !isbHaveSysMountPoint && "1" == sCanRemove) {
                // 如果该设备分区中没有挂载系统路径并且可移除，则卸载并弹出
                for (int i = 0; i < blkInfoList.count(); i++) {
                    QString cmd = "sudo udisksctl unmount -b " + blkInfoList.at(i).devPath;
                    proc->start(cmd);
                    qDebug() << cmd;
                    proc->waitForFinished();
                }

                // 判断当前设备信息列表是否为空
                if (!blkInfoList.isEmpty()) {
                    QString cmd = "sudo udisksctl power-off -b " + blkInfoList.last().parentDevPath;
                    proc->start(cmd);
                    qDebug() << cmd;
                    proc->waitForFinished();
                }
            }

            blkInfoList.clear();
            isbHaveSysMountPoint = false;
            parentDevPath = infoSecondCut.at(1);
            sCanRemove = infoSecondCut.at(3);
        }

        if (4 != infoSecondCut.count()) continue;

        // 获取分区dev节点
        QString devPath = infoSecondCut.at(1);
        //将分区信息存入列表
        BlkInfo blkInfo = {};
        blkInfo.type = type;
        // 获取分区dev节点
        blkInfo.devPath = infoSecondCut.at(1);
        // 获取主设备节点
        blkInfo.parentDevPath = parentDevPath;
        // 获取挂载节点
        blkInfo.mountPoint = infoSecondCut.at(2);
        // 判断该设备分区挂载路径中是否存在系统路径
        if (!blkInfo.mountPoint.startsWith("/media/") && !blkInfo.mountPoint.isEmpty()) {
            // 不在/media目录 和 非空，判定为系统路径
            isbHaveSysMountPoint = true;
        }

        // 获取是否可移除属性，与主设备保持一致
        blkInfo.canRemove = sCanRemove;
        qDebug() << "blkInfo:" << blkInfo.type << blkInfo.devPath << blkInfo.mountPoint << blkInfo.canRemove;
        blkInfoList.append(blkInfo);
    }

    proc->deleteLater();
}

void UsbManageMonitor::preManage()
{
    // 判断是否处于授权管理激活审核模式
    if (isActivatorInVerifingModel()) {
        return;
    }

    // 如果处于允许所有usb设备连接模式，则不进行u盘弹出操作
    if (USB_LIMIT_ALL == m_sLimitModel) {
        return;
    }

    disconnectAllDevices();

//    if (USB_LIMIT_ALL == m_sLimitModel) {
//        return;
//    } else if (USB_LIMIT_ZERO == m_sLimitModel) {
//        disconnectAllDevices();
//        return;
//    }

//    // 若为仅白名单模式
//    // 获取所有总线设备名称
//    QProcess proc;
//    QString cmd = "find /dev/bus";
//    proc.start(cmd);
//    proc.waitForFinished();

//    QString allInfoStr = proc.readAllStandardOutput().trimmed();
//    qDebug() << "allInfoStr" << allInfoStr;

//    QStringList firstCutStrList = allInfoStr.split("\n");
//    qDebug() << firstCutStrList;

//    // 遍历设备，并进行管控
//    for (QString sDevName : firstCutStrList) {
//        // 有总线设备名称使用 "udevadm info"命令得到设备信息
//        cmd = "udevadm info " + sDevName;
//        proc.start(cmd);
//        proc.waitForFinished();

//        QString sDevInfos = proc.readAllStandardOutput().trimmed();

//        QStringList sSecondCutStrList = sDevInfos.split("\n");
//        // 如果没有读到数据，则读取下一个
//        if (2 > sSecondCutStrList.size()) {
//            continue;
//        }

//        // 获取设备的 设备路径、型号、经销商id、制造商id、序列号、usb设备接口类型
//        QString sDevPath = "";
//        QString sVendor = "";
//        QString sVendorId = "";
//        QString sProductId = "";
//        QString sSerial = "";
//        QString sUsbInferfaces = "";
//        for (QString sInfo : sSecondCutStrList) {
//            if (sInfo.startsWith("E: DEVPATH=")) {
//                sDevPath = sInfo.split("=").last();
//            }
//            if (sInfo.startsWith("E: ID_VENDOR=")) {
//                sVendor = sInfo.split("=").last();
//            }
//            if (sInfo.startsWith("E: ID_VENDOR_ID=")) {
//                sVendorId = sInfo.split("=").last();
//            }
//            if (sInfo.startsWith("E: ID_MODEL_ID=")) {
//                sProductId = sInfo.split("=").last();
//            }
//            if (sInfo.startsWith("E: ID_SERIAL_SHORT=")) {
//                sSerial = sInfo.split("=").last();
//            }

//            if (sInfo.startsWith("E: ID_USB_INTERFACES=")) {
//                // 除去符号-":"，并且只保留开头两个字符
//                sUsbInferfaces = sInfo.split("=").last().remove(":").left(2);
//            }
//        }
//        qDebug() << "------dev infos" << sDevPath << sVendor << sVendorId << sProductId << sSerial << sUsbInferfaces;

//        // 只对存储设备进行管控
//        if ("06" != sUsbInferfaces &&
//            "08" != sUsbInferfaces && "ff" != sUsbInferfaces) {
//            continue;
//        }

//        // 首次启动按白名单对usb设备进行断开处理
//        if (0 != checkWhitelist(sSerial, sVendorId, sProductId)) {
//            disconnectDevice("/sys" + sDevPath);
//        }
//    }
//    // end======遍历设备，并进行管控

    //    proc.deleteLater();
}

// 判断是否处于授权管理激活审核模式
bool UsbManageMonitor::isActivatorInVerifingModel()
{
    bool ret = false;
    // 激活审核模式路径
    QString verifingModelFilePath = QString("/deepin-installer/lightdm-stop.sh");
    QFile f(verifingModelFilePath);

    // 如果存在，则为审核模式
    ret = f.exists();
    return ret;
}

void UsbManageMonitor::writeConnectionRecord(QString name, QString sDatetime, QString status,
                                             QString serial, QString vendorID, QString productID,
                                             QString sParentSize, QString sSize)
{
    // 如果记录开关未打开, 则不写入记录
    if (!m_isbSaveRecord) {
        return;
    }

    //name--time--isbBlocked--serial--vendorID--productID--sParentSize--sSize
    QDir dir;
    //目录不存在，就创建
    if (!dir.exists(m_configPath)) {
        dir.mkpath(m_configPath);
    }

    QString m_connectionLogPath = m_configPath + QString("usbconnection.log");
    QFile f(m_connectionLogPath);
    if (!f.open(QIODevice::ReadOnly | QIODevice::WriteOnly | QIODevice::Text)) {
        qDebug() << "Open usbconnection.log failed." << endl;
        return;
    }

    QTextStream txtOutput(&f);
    txtOutput.seek(f.size());

    txtOutput << name << "," << sDatetime << ","  << status << ","
              << serial << ","  << vendorID << ","  << productID << ','
              << sParentSize << ","  << sSize << '\n';

    // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
    int fd = f.handle();
    fsync(fd);
    f.close();

    UsbConnectionInfo info;
    info.sName = name;
    info.sDatetime = sDatetime;
    info.sStatus = status;
    info.sSerial = serial;
    info.sVendorID = vendorID;
    info.sProductID = productID;
    info.sParentSize = sParentSize;
    info.sSize = sSize;
    m_usbConnectionInfoLst.append(info);
    // debug
    qDebug() << "writeConnectionRecord" << info.sName << info.sStatus << info.sSerial << info.sSize;

    //修改日志文件权限
    QProcess proc;
    proc.start(QString("sudo chmod 777 %1").arg(m_connectionLogPath));
    proc.waitForFinished();
    proc.deleteLater();
}

// 备份原有数据（在每次清除连接记录之前必须调用）
void UsbManageMonitor::backupUsbConnectionLog()
{
    // 本地记录文件
    QDir dir;
    // 目录不存在，就创建
    if (!dir.exists(m_configPath)) {
        dir.mkpath(m_configPath);
    }

    QString m_connectionLogPath = m_configPath+ QString(USB_CONNECTION_LOG_FILE_NAME);
    QFile f(m_connectionLogPath);

    // 为避免数据丢失，清空之前先备份原有数据
    if (f.exists()) {
        f.rename(m_configPath+ QString(USB_CONNECTION_LOG_BAK_FILE_NAME));
        // 将数据同步到磁盘，防止强制断电损坏文件
        int fd = f.handle();
        fsync(fd);
    }
}

// 清除备份数据（在写完所有数据后必须调用）
void UsbManageMonitor::clearBackupOfUsbConnectionLog()
{
    // 本地记录文件
    QDir dir;
    // 目录不存在，就创建
    if (!dir.exists(m_configPath)) {
        dir.mkpath(m_configPath);
    }

    QString m_connectionLogPath = m_configPath+ QString(USB_CONNECTION_LOG_BAK_FILE_NAME);
    QFile f(m_connectionLogPath);

    // 为避免数据都是，清空之前先备份原有数据
    if (f.exists()) {
        f.remove();
        // 将数据同步到磁盘，防止强制断电损坏文件
        int fd = f.handle();
        fsync(fd);
    }
}

void UsbManageMonitor::clearUsbConnectionLog()
{
    //清除本地记录文件
    QDir dir;
    //目录不存在，就创建
    if (!dir.exists(m_configPath)) {
        dir.mkpath(m_configPath);
    }

    QString m_connectionLogPath = m_configPath+ QString("usbconnection.log");
    QFile f(m_connectionLogPath);
    if (!f.open(QIODevice::WriteOnly | QIODevice::Text)) {
        qDebug() << "Open usbconnection.log failed." << endl;
        return;
    }

    // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
    int fd = f.handle();
    fsync(fd);
    f.close();
    m_usbConnectionInfoLst.clear();

    // debug
    qDebug() << "clearUsbConnectionLog";

    //修改文件权限
    QProcess proc;
    proc.start(QString("sudo chmod 777 %1").arg(m_connectionLogPath));
    proc.waitForFinished();
    proc.deleteLater();
}

void UsbManageMonitor::cleanUsbConnectionLog()
{
    // 如果记录开关未打开, 则不清理日志
    if (!m_isbSaveRecord) {
        return;
    }

    QList<UsbConnectionInfo> usbConnectionInfoLstTmp = m_usbConnectionInfoLst;
    // 清空数据之前，先备份数据
    backupUsbConnectionLog();
    clearUsbConnectionLog();
    QDateTime limitTime = QDateTime::currentDateTime().addDays(-30);
    for (UsbConnectionInfo info : usbConnectionInfoLstTmp) {
        QDateTime infoTime = QDateTime::fromString(info.sDatetime, "yyyy-MM-dd hh:mm:ss");
        //只记录30天以内的连接数据
        if (limitTime < infoTime) {
            writeConnectionRecord(info.sName, info.sDatetime, info.sStatus, info.sSerial,
                                  info.sVendorID ,info.sProductID, info.sParentSize, info.sSize);
        }
    }

    // 数据写入完毕后，清除备份文件
    clearBackupOfUsbConnectionLog();
}

void UsbManageMonitor::writeConnectionWhiteLst(QString name, QString sDatetime, QString status,
                                               QString serial, QString vendorID, QString productID,
                                               QString sParentSize, QString sSize)
{
    //name--time--isbBlocked--serial--vendorID--productID--sParentSize--sSize
    QDir dir;
    //目录不存在，就创建
    if (!dir.exists(m_configPath)) {
        dir.mkpath(m_configPath);
    }

    QString m_whitelistPath = m_configPath + QString("usbconnectionwhitelist.list");
    QFile f(m_whitelistPath);
    if (!f.open(QIODevice::ReadOnly | QIODevice::WriteOnly | QIODevice::Text)) {
        qDebug() << "Open usbconnectionwhitelist.list failed." << endl;
        return;
    }

    QTextStream txtOutput(&f);
    txtOutput.seek(f.size());

   QString sSavingInfo;
   sSavingInfo = name + "," + sDatetime + ","  + status + ","
               + serial + ","  + vendorID + ","  + productID + ','
               + sParentSize + ","  + sSize + '\n';
   // debug
   qDebug() << "writeConnectionWhiteLst" << sSavingInfo;

    //加密
    f.write(encryptString(sSavingInfo, f.size()));

    // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
    int fd = f.handle();
    fsync(fd);
    f.close();

    UsbConnectionInfo info;
    info.sName = name;
    info.sDatetime = sDatetime;
    info.sStatus = status;
    info.sSerial = serial;
    info.sVendorID = vendorID;
    info.sProductID = productID;
    info.sParentSize = sParentSize;
    info.sSize = sSize;
    m_whiteList.append(info);

    //修改日志文件权限
    QProcess proc;
    proc.start(QString("sudo chmod 777 %1").arg(m_whitelistPath));
    proc.waitForFinished();
    proc.deleteLater();
}

// 备份白名单原有数据（在每次清除白名单之前必须调用）
void UsbManageMonitor::backupConnectionWhiteLst()
{
    // 本地记录文件
    QDir dir;
    // 目录不存在，就创建
    if (!dir.exists(m_configPath)) {
        dir.mkpath(m_configPath);
    }

    QString m_whiteLstPath = m_configPath+ QString(USB_WHITELIST_FILE_NAME);
    QFile f(m_whiteLstPath);

    // 为避免数据丢失，清空之前先备份原有数据
    if (f.exists()) {
        f.rename(m_configPath+ QString(USB_WHITELIST_BAK_FILE_NAME));
        // 将数据同步到磁盘，防止强制断电损坏文件
        int fd = f.handle();
        fsync(fd);
        qDebug() << "============= backup" << USB_WHITELIST_FILE_NAME;
    }
}

// 清除白名单备份数据（在写完所有白名单数据后必须调用）
void UsbManageMonitor::clearBackupOfConnectionWhiteLst()
{
    // 本地记录文件
    QDir dir;
    // 目录不存在，就创建
    if (!dir.exists(m_configPath)) {
        dir.mkpath(m_configPath);
    }

    QString m_connectionLogPath = m_configPath+ QString(USB_WHITELIST_BAK_FILE_NAME);
    QFile f(m_connectionLogPath);

    // 为避免数据都是，清空之前先备份原有数据
    if (f.exists()) {
        f.remove();
        // 将数据同步到磁盘，防止强制断电损坏文件
        int fd = f.handle();
        fsync(fd);
    }
}

void UsbManageMonitor::clearConnectionWhiteLst()
{
    //清除本地白名单
    QDir dir;
    //目录不存在，就创建
    if (!dir.exists(m_configPath)) {
        dir.mkpath(m_configPath);
    }
    QString m_whitelistPath = m_configPath + QString("usbconnectionwhitelist.list");
    QFile f(m_whitelistPath);
    if (!f.open(QIODevice::WriteOnly | QIODevice::Text)) {
        qDebug() << "Open usbconnection.log failed." << endl;
        return;
    }

    // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
    int fd = f.handle();
    fsync(fd);
    f.close();
    m_whiteList.clear();

    // debug
    qDebug() << "clear usbconnectionwhitelist";

    //修改文件权限
    QProcess proc;
    proc.start(QString("sudo chmod 777 %1").arg(m_whitelistPath));
    proc.waitForFinished();
    proc.deleteLater();
}

void UsbManageMonitor::removeFromWhiteLst(QString sSerial, QString sVendorID, QString sProductID)
{
    QList<UsbConnectionInfo> lastUsbWhiteList = m_whiteList;
    // 清空数据之前，先备份数据
    backupConnectionWhiteLst();
    clearConnectionWhiteLst();

    for (UsbConnectionInfo info : lastUsbWhiteList) {
        if (sSerial == info.sSerial && sVendorID == info.sVendorID && sProductID == info.sProductID) {
            continue;
        }
        writeConnectionWhiteLst(info.sName, info.sDatetime, info.sStatus, info.sSerial,
                                info.sVendorID ,info.sProductID, info.sParentSize, info.sSize);
    }
    // 清除白名单备份数据
    clearBackupOfConnectionWhiteLst();
}

void UsbManageMonitor::changeIsbSaveRecord(bool change)
{
    m_isbSaveRecord = change;
    if (change) {
        m_config->setValue("SaveRecordSwitch", "true");
    } else {
        m_config->setValue("SaveRecordSwitch", "false");
    }
    m_config->sync();

}

void UsbManageMonitor::selectLimitationModel(QString modelStr)
{
    m_sLimitModel = modelStr;
    m_config->setValue("LimitaionModel", modelStr);
    m_config->sync();

    // debug
    qDebug() << "selectLimitationModel" << modelStr;
}

QString UsbManageMonitor::getUsbConnectionRecords()
{
    QString sConnectionRecords;
    for (UsbConnectionInfo info : m_usbConnectionInfoLst) {
        sConnectionRecords += info.sName +  (",") + info.sDatetime + (",") + info.sStatus + (",") +
                info.sSerial +  (",") + info.sVendorID + (",") + info.sProductID + (",") +
                info.sParentSize + (",") + info.sSize + ("\n");
    }
    return sConnectionRecords;
}

QString UsbManageMonitor::getWhiteList()
{
    QString sWhiteList;
    for (UsbConnectionInfo info : m_whiteList) {
        sWhiteList += info.sName +  (",") + info.sDatetime + (",") + info.sStatus + (",") +
                info.sSerial +  (",") + info.sVendorID + (",") + info.sProductID + (",") +
                info.sParentSize + (",") + info.sSize + ("\n");
    }
    return sWhiteList;
}

void UsbManageMonitor::monitorDevices(struct udev *udev)
{
    // 使用udev_monitor_new_from_netlink创建一个新的monitor，
    // 函数的第二个参数是事件源的名称，可选"kernel"或"udev"。
    // 基于"kernel"的事件通知要早于"udev"，但相关的设备结点未必创建完成，
    // 所以一般应用的设计要基于"udev"进行监控
    struct udev_monitor *mon = udev_monitor_new_from_netlink(udev, "udev");

    // 使用udev_monitor_filter_add_match_subsystem_devtype增加一个基于设备类型的udev事件过滤器
    udev_monitor_filter_add_match_subsystem_devtype(mon, /*SUBSYSTEM*/ "block", nullptr);
    udev_monitor_filter_add_match_subsystem_devtype(mon, /*SUBSYSTEM*/ "usb", nullptr);
    // 使用udev_monitor_enable_receiving启动监控过程
    udev_monitor_enable_receiving(mon);

    // 监控可以使用udev_monitor_get_fd获取一个文件描述符，基于返回的fd可以执行poll操作，简化程序设计
    int fd = udev_monitor_get_fd(mon);

    while (m_status) {
        fd_set fds;
        FD_ZERO(&fds);
        FD_SET(fd, &fds);

        // Wait for monitor to return
        int ret = select(fd + 1, &fds, nullptr, nullptr, nullptr);
//        if (ret <= 0) {
//            break;
//        }

        if (ret > 0) {
            if (FD_ISSET(fd, &fds)) {
                // 插拔事件到达后，可以使用udev_monitor_receive_device获取产生事件的设备映射
                struct udev_device *dev = udev_monitor_receive_device(mon);
                processDevice(dev);
            }
        }
    }
}

void UsbManageMonitor::processDevice(struct udev_device *dev)
{
    qDebug() << "**********************************************";
    //设备路径
    const char *sysPathTmp = udev_device_get_syspath(dev);
    printf("Devicesyspath: %s\n", sysPathTmp);
    const char *devPath = udev_device_get_devpath(dev);
    printf("Device dev path:%s\n", udev_device_get_devpath(dev));
    //设备行为
    const char *action = udev_device_get_action(dev);
    printf("Deviceaction: %s\n", action);
    //子系统
    const char *subSystem = udev_device_get_subsystem(dev);
    printf("Device subsystem:%s\n", subSystem);
    //设备类型
    const char *devType = udev_device_get_devtype(dev);
    printf("Devicedevtype: %s\n", devType);
    //设备节点
    const char *devNode = udev_device_get_devnode(dev);
    printf("DeviceNode Path: %s\n", devPath);

    // 如果添加 或 改变 了设备
    if (0 == strcmp("add", action) ||
        0 == strcmp("change", action) ) {
        // 且设备系统路径前缀不同，说明添加新的不同设备
        // 如果添加了新设备，清空之前的设备信息
        if (!QString(sysPathTmp).startsWith(m_usbDeviceInfo.sSysPath)) {
            qDebug() << "new usb device add";
            m_usbDeviceInfo = {};
            m_usbDeviceInfo.sSysPath = sysPathTmp;

            m_isbManagedDiskStorage = false;
            m_isbManagedMobileStorage = false;
            m_disConnectFlag = 1;
        }
    }
    qDebug() << "usb device sys path: " << m_usbDeviceInfo.sSysPath;

    /*
    // 获取主设备系统路径
    QStringList sysPathRealLst = QString(sysPathTmp).split("/");
    QString sysPathReal = "";
    if (!sysPathRealLst.isEmpty()) {
        sysPathRealLst.removeFirst();
        sysPathRealLst.removeLast();

        for (QString sSysPathPiece : sysPathRealLst) {
            sysPathReal += "/" + sSysPathPiece;
        }

//        sysPathReal =  QString("/") + sysPathRealLst.at(1) + QString("/") + sysPathRealLst.at(2) + QString("/") + sysPathRealLst.at(3) + QString("/") +
//                               sysPathRealLst.at(4) + QString("/") + sysPathRealLst.at(5) + QString("/") + sysPathRealLst.at(6);

        //如果添加了新设备，清空之前的设备信息
        if (m_usbDeviceInfo.sSysPath != sysPathReal) {
            m_usbDeviceInfo = {};
            m_isbManagedDiskStorage = false;
            m_isbManagedMobileStorage = false;
            m_disConnectFlag = 1;
        }
        m_usbDeviceInfo.sSysPath = sysPathReal;
        qDebug() << "usb device sys path: " << m_usbDeviceInfo.sSysPath;
    }
    */

    // 如果设备移除，并且是根设备
    if ("remove" == QString(action) &&
        sysPathTmp == m_usbDeviceInfo.sSysPath
        ) {
        //清空设备信息
        m_usbDeviceInfoStr = "";
        m_usbDeviceInfo = {};
        m_usbDeviceInfo.sSysPath = "*";
        m_isbManagedDiskStorage = false;
        m_isbManagedMobileStorage = false;
        m_disConnectFlag = 1; //0为断开状态，1为非断开状态
        return;
    }

    // 使用父设备结构获取基本信息
    const char *product = udev_device_get_sysattr_value(dev, "product");
    qDebug() << "udev_device_get_sysattr_value product: " << product;

    const char *serial = udev_device_get_sysattr_value(dev, "serial");
    qDebug() << "udev_device_get_sysattr_value serial: " << serial;

    const char *idVendor = udev_device_get_sysattr_value(dev, "idVendor");
    qDebug() << "udev_device_get_sysattr_value idVendor: " << idVendor;

    const char *idProduct = udev_device_get_sysattr_value(dev, "idProduct");
    qDebug() << "udev_device_get_sysattr_value idProduct: " << idProduct;

    // 使用新加子设备结构获取当前子设备信息
    //设备类 获取值代表 "03"-鼠标 "08"-usb存储设备/手机内置光盘 "ff"-手机存储设备(MTP) "06"-手机相机存储(PTP)
    const char *bInterfaceClass = udev_device_get_sysattr_value(dev, "bInterfaceClass");
    qDebug() << "udev_device_get_sysattr_value bInterfaceClass: " << bInterfaceClass;

    const char *bcdUSB = udev_device_get_sysattr_value(dev, "bcdUSB");
    qDebug() << "udev_device_get_sysattr_value bcdUSB: " << bcdUSB;

    // bus总线序号, 对应与/dev/bus/usb/[busnum]/[devnum],
    // 可使用udevadm info /dev/bus/usb/[busnum]/[devnum]命令查看设备信息
    const char *busnum = udev_device_get_sysattr_value(dev, "busnum");
    qDebug() << "udev_device_get_sysattr_value busnum: " << busnum;

    // 设备序号
    const char *devnum = udev_device_get_sysattr_value(dev, "devnum");
    qDebug() << "udev_device_get_sysattr_value devnum: " << devnum;

    //更新设备信息
    // 型号
    if (!QString(product).isEmpty()) {
        m_usbDeviceInfo.sProduct = product;
    }
    // 序列号
    if (!QString(serial).isEmpty()) {
        m_usbDeviceInfo.sSerial = serial;
    }
    // 生产商id
    if (!QString(idVendor).isEmpty()) {
        m_usbDeviceInfo.sIdVendor = idVendor;
    }
    // 产品id
    if (!QString(idProduct).isEmpty()) {
        m_usbDeviceInfo.sIdProduct = idProduct;
    }
    // 接口类型
    if (!QString(bInterfaceClass).isEmpty()) {
        m_usbDeviceInfo.sInterfaceClass = bInterfaceClass;
    }
    // bus总线号
    if (!QString(busnum).isEmpty()) {
        m_usbDeviceInfo.sBusnum = busnum;
    }
    // 设备号
    if (!QString(devnum).isEmpty()) {
        m_usbDeviceInfo.sDevNum = devnum;
    }
    // 设备/sys下的路径
    if (!QString(devPath).isEmpty()) {
        m_usbDeviceInfo.sDevPath = devPath;
    }

    // 只管控usb存储设备
    //设备类 获取值代表 "03"-鼠标 "08"-usb存储设备/手机内置光盘 "ff"-手机存储设备(MTP) "06"-手机相机存储(PTP)
    if ("06" != m_usbDeviceInfo.sInterfaceClass &&
        "ff" != m_usbDeviceInfo.sInterfaceClass &&
        "08" != m_usbDeviceInfo.sInterfaceClass) {
        return;
    }

    if (0 == m_disConnectFlag) {
        disconnectDevice(m_usbDeviceInfo.sSysPath);
        qDebug() << "repeat disconnecting device";
        return;
    }

    if (false == m_isbManagedDiskStorage) {
        qDebug() << "m_usbDeviceInfo.bInterfaceClass:" << m_usbDeviceInfo.sInterfaceClass;
        //设备类型为 存储设备时，进行管控
        //设备类 获取值代表 "03"-鼠标 "08"-usb存储设备/手机内置光盘 "ff"-手机存储设备(MTP) "06"-手机相机存储(PTP)
        if ("08" == m_usbDeviceInfo.sInterfaceClass) {
            m_isbManagedDiskStorage = true;
            manageDiskStorage();
        }
    }

    if (false == m_isbManagedMobileStorage) {
        qDebug() << "m_usbDeviceInfo.bInterfaceClass:" << m_usbDeviceInfo.sInterfaceClass;
        //设备类型为 存储设备时，进行管控
        //设备类 获取值代表 "03"-鼠标 "08"-usb存储设备/手机内置光盘 "ff"-厂商定义的设备(包括MTP) "06"-手机相机存储(PTP)
        if ("ff" == m_usbDeviceInfo.sInterfaceClass &&
            isbMtpDevice(m_usbDeviceInfo.sBusnum, m_usbDeviceInfo.sDevNum)) {
            // 如果是mtp设备,则进行管控
            m_isbManagedMobileStorage = true;
            manageMobileStorage();
        } else if ("06" == m_usbDeviceInfo.sInterfaceClass) {
            // 如果是手机相机存储
            m_isbManagedMobileStorage = true;
            manageMobileStorage();
        }
    }

    // 获取父设备大小
    if ("disk" == QString(devType)) {
        m_usbDeviceInfo.sParentSize = getDevSize(devNode);
    }

    // 获取磁盘或磁盘分区的卷标
    if ("disk" == QString(devType) ||
        "partition" == QString(devType)) {
        if (devNode != m_usbDeviceInfo.sDevNode) {
            m_usbDeviceInfo.sDevNode = devNode;
            m_usbDeviceInfo.sLabel = getDevLabel(devNode);
            m_usbDeviceInfo.sSize = getDevSize(devNode);

            // 如果获取此分区大小为空，则返回，不处理
            if (m_usbDeviceInfo.sSize.isEmpty())
                return;
            // 如果此分区文件系统为空，则返回不处理
            if (getDevFsType(devNode).isEmpty())
                return;

            m_usbDeviceInfoStr = mergeDevSavingInfoToStr(m_usbDeviceInfo.sLabel, "Connected", m_usbDeviceInfo.sSerial,
                                                         m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                                         m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
            Q_EMIT sendConnectionRecord(m_usbDeviceInfoStr);
            // clean
            cleanUsbConnectionLog();
            // updateNameInWhiteLst
            updateNameInWhiteLst(m_usbDeviceInfo.sLabel, m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
            // 写入记录
            QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
            writeConnectionRecord(m_usbDeviceInfo.sLabel, sDatetime, "Connected", m_usbDeviceInfo.sSerial,
                                  m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                  m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
        }
    }

    //mobile usb cd rom - get mount point
    if ("disk" == QString(devType)) {
        // if is not the same dev node
        if (devNode != m_usbDeviceInfo.sDevNode) {
            m_usbDeviceInfo.sDevNode = devNode;
            QString romMountPointTailStr = QString(devNode).split("/").last();
            if (romMountPointTailStr.size() < 3) {
                qDebug() << "not get a device node" << romMountPointTailStr;
            } else {
                if (QString("s") == romMountPointTailStr.at(0) && QString("r") == romMountPointTailStr.at(1)) {
                    qDebug() << "get a usb rom device node" << romMountPointTailStr;
                    m_usbDeviceInfo.sLabel = getDevLabel(devNode);

                    // 如果获取此分区的挂载点为空，则返回，不处理
                    if (m_usbDeviceInfo.sLabel.isEmpty())
                        return;

                    m_usbDeviceInfoStr = mergeDevSavingInfoToStr(m_usbDeviceInfo.sLabel, "Connected", m_usbDeviceInfo.sSerial,
                                                                 m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                                                 m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
                    Q_EMIT sendConnectionRecord(m_usbDeviceInfoStr);
                    // clean
                    cleanUsbConnectionLog();
                    // updateNameInWhiteLst
                    updateNameInWhiteLst(m_usbDeviceInfo.sLabel, m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
                    // 写入记录
                    QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
                    writeConnectionRecord(m_usbDeviceInfo.sLabel, sDatetime, "Connected", m_usbDeviceInfo.sSerial,
                                          m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                          m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
                }
            }
        } //end - if is not the same dev node
    }

}

int UsbManageMonitor::checkWhitelist(QString sSerial, QString sIdVendor, QString sIdProduct)
{
    if (sSerial.isEmpty() || sIdVendor.isEmpty() || sIdProduct.isEmpty()) {
        return 1;
    }

    for (UsbConnectionInfo info : m_whiteList) {
        if (sSerial == info.sSerial && sIdVendor == info.sVendorID && sIdProduct == info.sProductID) {
            //如果设备在白名单中返回false
            return 0;
        }
    }
    return 1;
}

int UsbManageMonitor::disconnectDevice(struct udev_device *dev)
{
    // Open file descriptor to the bus unbind interface
    int file = open("/sys/bus/usb/drivers/usb/unbind", O_WRONLY, 0);


    if (file == -1) {
        qDebug() << "open \'unbind\' file failed !";
        return 1;
    }
    qDebug() << "open \'unbind\' file success !";

    // Grab the path to the device
    const char *path = udev_device_get_devpath(dev);

    char *token;
    char *str = strdup(path);
    char *location;


    if (!path) {
        // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
        fsync(file);
        close(file);
        qDebug() << "devpath is empty!";
        return 1;
    }

    // Grab only the relevant bus location from the path
    while ((token = strsep(&str, "/")) != NULL) {
        location = strdup(token);
    }

    // create the control block structure
    // and buffer to write
    struct aiocb cb;
    char write_buffer[strlen(location) + 1];

    memset(&cb, 0, sizeof(struct aiocb));
    memcpy(write_buffer, location, strlen(location) + 1);
    cb.aio_nbytes = strlen(location) + 1;
    cb.aio_fildes = file;
    cb.aio_offset = 0;
    cb.aio_buf = write_buffer;

    //printf("%lu\n", strlen(write_buffer));
    //printf("%s\n", write_buffer);

    // Async-write this location to the unbind interface
    if (aio_write(&cb) == -1) {
        // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
        fsync(file);
        close(file);
        qDebug() << "Async write failed!";
        return 1;
    }

    /* Wait until completion */
    while (aio_error(&cb) == EINPROGRESS);

    // Check for errors
    int err = aio_error(&cb); //错误返回0
//    int ret = aio_return(&cb);

    if (err == 0) {
        // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
        fsync(file);
        close(file);
        qDebug() << "usb disconnect error!";
        return 0;
    }

    // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
    fsync(file);
    close(file);

    qDebug() << "disconnect usb device!";
    return 0;

}

int UsbManageMonitor::disconnectDevice(QString sysPath)
{
    std::string sysPathStdString = sysPath.toStdString();
    const char *sysPathChar = sysPathStdString.c_str();
    qDebug() << "sysPath" << sysPath;
    qDebug() << "sysPathChar" << sysPathChar;
    struct udev_device *dev = udev_device_new_from_syspath(m_udev, sysPathChar);

    // Open file descriptor to the bus unbind interface
    int file = open("/sys/bus/usb/drivers/usb/unbind", O_WRONLY, 0);


    if (file == -1) {
        qDebug() << "open \'unbind\' file failed !";
        return 1;
    }
    qDebug() << "open \'unbind\' file success !";

    // Grab the path to the device
    const char *path = udev_device_get_devpath(dev);
    if (QString(path).size() < 1) {
        qDebug() << "devpath is error, disconnect failed!";
        return 1;
    }
    qDebug() << "disconnecting devpath" << path;

    char *token;
    char *str = strdup(path);
    char *location;


    if (!path) {
        // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
        fsync(file);
        close(file);
        qDebug() << "devpath is empty!";
        return 1;
    }

    // Grab only the relevant bus location from the path
    while ((token = strsep(&str, "/")) != NULL) {
        location = strdup(token);
    }

    // create the control block structure
    // and buffer to write
    struct aiocb cb;
    char write_buffer[strlen(location) + 1];

    memset(&cb, 0, sizeof(struct aiocb));
    memcpy(write_buffer, location, strlen(location) + 1);
    cb.aio_nbytes = strlen(location) + 1;
    cb.aio_fildes = file;
    cb.aio_offset = 0;
    cb.aio_buf = write_buffer;

    //printf("%lu\n", strlen(write_buffer));
    //printf("%s\n", write_buffer);

    // Async-write this location to the unbind interface
    if (aio_write(&cb) == -1) {
        // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
        fsync(file);
        close(file);
        qDebug() << "Async write failed!";
        return 1;
    }

    /* Wait until completion */
    while (aio_error(&cb) == EINPROGRESS);

    // Check for errors
    int err = aio_error(&cb); //错误返回0
//    int ret = aio_return(&cb);

    if (err == 0) {
        // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
        fsync(file);
        close(file);
        qDebug() << "usb disconnect error!";
        return 0;
    }

    // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
    fsync(file);
    close(file);

    qDebug() << "disconnect usb device!";
    return 0;
}

QString UsbManageMonitor::getDevMountPoint(QString devNode)
{
    QString mountPointStr;

    QProcess *getDevNameProc = new QProcess(this);
    QString cmd = "lsblk -no MOUNTPOINT "; // sudo blkid -s LABEL -o value, lsblk -no MOUNTPOINT
    cmd.append(devNode);
    getDevNameProc->start(cmd);
    qDebug() << cmd;
    getDevNameProc->waitForFinished();

    mountPointStr = getDevNameProc->readAllStandardOutput().trimmed();
    qDebug() << "--------mountPointStr" << mountPointStr;

    getDevNameProc->deleteLater();
    return mountPointStr;
}

QString UsbManageMonitor::getDevLabel(QString sDevNode)
{
    QString sLabelStr;

    QProcess getDevLabelProc;
    QString cmd = "sudo blkid -d -s LABEL -o value "; // blkid -d -s LABEL -o value, sudo lsblk -no LABEL
    cmd.append(sDevNode);
    getDevLabelProc.start(cmd);
    qDebug() << cmd;
    getDevLabelProc.waitForFinished();

    sLabelStr = getDevLabelProc.readAllStandardOutput().trimmed();
    qDebug() << "--------LABEL" << sLabelStr;

    getDevLabelProc.deleteLater();
    return sLabelStr;
}

QString UsbManageMonitor::getDevSize(QString sDevNode)
{
    QString sParentSizeStr;
    QProcess getDevNameProc;
    QString sCmd = "lsblk -no SIZE ";
    sCmd.append(sDevNode);
    getDevNameProc.start(sCmd);
    qDebug() << sCmd;
    getDevNameProc.waitForFinished();

    sParentSizeStr = getDevNameProc.readAllStandardOutput().trimmed().split('\n').first();
    sParentSizeStr.append("B");
    qDebug() << "--------sDevSizeStr" << sParentSizeStr;

    return sParentSizeStr;
}

// 获取设备文件系统类型
QString UsbManageMonitor::getDevFsType(QString sDevNode)
{
    QString sFsType;
    QProcess getDevFsTypeProc;
    QString cmd = "lsblk -no FSTYPE ";
    cmd.append(sDevNode);
    getDevFsTypeProc.start(cmd);
    qDebug() << cmd;
    getDevFsTypeProc.waitForFinished();

    QString sAllFsType = getDevFsTypeProc.readAllStandardOutput();

    // 只获取第一个数据
    sFsType = sAllFsType.split("\n").first().trimmed();
    qDebug() << "--------sFsType" << sFsType;

    getDevFsTypeProc.deleteLater();
    return sFsType;
}

// 是否是mtp设备
bool UsbManageMonitor::isbMtpDevice(QString sBusNum, QString sDevNum)
{
    // 待返回值
    // 是否是mtp设备
    bool isbMtpDeviceRet = false;
    // mtp设备id
    QString sIdMtpDevice = "";

    // 按照3位数格式化bus总线号
    int iBusNum = sBusNum.toInt();
    QString sFormatBusNum = QString("%1").arg(iBusNum, 3, 10, QChar('0'));

    // 按照3位数格式化设备号
    int iDevNum = sDevNum.toInt();
    QString sFormatDevNum = QString("%1").arg(iDevNum, 3, 10, QChar('0'));

    // 组成设备路径
    QString sDevicePath = QString("/dev/bus/usb/")
            .append(sFormatBusNum).append("/")
            .append(sFormatDevNum);

    // 使用 udevadm info 读取设备信息
    QProcess proc;
    // 命令
    QString sCmd = "udevadm info " + sDevicePath;
    proc.start(sCmd);
    qDebug() << sCmd;
    // 查询等待限时50ms
    proc.waitForFinished(50);
    QString sDevInfos = proc.readAllStandardOutput().trimmed();
    // 分割每行数据
    QStringList sFirstCutStrList = sDevInfos.split("\n");
    // 获取mtp设备id
    for (QString sDevInfo : sFirstCutStrList) {
        if (sDevInfo.startsWith("E: ID_MTP_DEVICE")) {
            sIdMtpDevice = sDevInfo.split("=").last();
            break;
        }
    }

    // 根据是否存在mtp设备id,判断是否是mtp设备
    if (!sIdMtpDevice.isEmpty()) {
        qDebug() << sDevicePath + " is mtp device";
        isbMtpDeviceRet = true;
    }

    return isbMtpDeviceRet;
}

// 将待发送设备信息整合成字符串
QString UsbManageMonitor::mergeDevSavingInfoToStr(QString sName, QString sStatus, QString sSerial, QString sVendorID,
                                                  QString sProductID, QString sParentSize, QString sSize)
{
    QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
    return sName +  (",") + sDatetime + (",") + sStatus + (",") + sSerial +  (",") +
            sVendorID + (",") + sProductID + (",") + sParentSize + (",") + sSize + ("\n");
}

void UsbManageMonitor::manageDiskStorage()
{
    qDebug() << "disk存储设备时，进行管控";
    m_usbDeviceInfoStr = mergeDevSavingInfoToStr(m_usbDeviceInfo.sProduct, "Blocked", m_usbDeviceInfo.sSerial,
                                                 m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                                 m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
    if (USB_LIMIT_ALL == m_sLimitModel) {
//            Q_EMIT checkUsbInsert(true, m_usbDeviceInfoStr);
    } else if (USB_LIMIT_ZERO == m_sLimitModel) {
        m_disConnectFlag = disconnectDevice(m_usbDeviceInfo.sSysPath);
        if (m_disConnectFlag == 0) {
            Q_EMIT sendConnectionRecord(m_usbDeviceInfoStr);
            Q_EMIT requestShowBlockAllSysNotify();
            // clean
            cleanUsbConnectionLog();
            // updateNameInWhiteLst
            updateNameInWhiteLst(m_usbDeviceInfo.sProduct, m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
            // 写入记录
            QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
            writeConnectionRecord(m_usbDeviceInfo.sProduct, sDatetime, "Blocked", m_usbDeviceInfo.sSerial,
                                  m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                  m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
            return;
        } else {
//                Q_EMIT checkUsbInsert(true, m_usbDeviceInfoStr);
        }
    } else if (USB_LIMIT_PARTY == m_sLimitModel) {
        int nFlag = checkWhitelist(m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
        if (nFlag == 0) {
//                Q_EMIT checkUsbInsert(true, m_usbDeviceInfoStr);
        } else {
            m_disConnectFlag = disconnectDevice(m_usbDeviceInfo.sSysPath);
            if (m_disConnectFlag == 0) {
                Q_EMIT sendConnectionRecord(m_usbDeviceInfoStr);
                Q_EMIT requestShowOnlyWhiteListSysNotify();
                // clean
                cleanUsbConnectionLog();
                // updateNameInWhiteLst
                updateNameInWhiteLst(m_usbDeviceInfo.sProduct, m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
                // 写入记录
                QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
                writeConnectionRecord(m_usbDeviceInfo.sProduct, sDatetime, "Blocked", m_usbDeviceInfo.sSerial,
                                      m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                      m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
                return;
            } else {
//                    Q_EMIT checkUsbInsert(true, m_usbDeviceInfoStr);
            }
        }
    } else {
//            Q_EMIT checkUsbInsert(true, m_usbDeviceInfoStr);
    }

}

void UsbManageMonitor::manageMobileStorage()
{
    qDebug() << "mobile存储设备时，进行管控";
    // 由于手机存储设备只存在"disk"类型，没有子分区，所以直接把主分区限制结果发送出去
    QString sBlockedUsbDeviceInfoStr = mergeDevSavingInfoToStr(m_usbDeviceInfo.sProduct, "Blocked", m_usbDeviceInfo.sSerial,
                                                 m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                                 m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
    QString sConnectedUsbDeviceInfoStr = mergeDevSavingInfoToStr(m_usbDeviceInfo.sProduct, "Connected", m_usbDeviceInfo.sSerial,
                                                 m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                                 m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);

    if (USB_LIMIT_ALL == m_sLimitModel) {
        Q_EMIT sendConnectionRecord(sConnectedUsbDeviceInfoStr);
        // clean
        cleanUsbConnectionLog();
        // updateNameInWhiteLst
        updateNameInWhiteLst(m_usbDeviceInfo.sProduct, m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
        // 写入记录
        QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
        writeConnectionRecord(m_usbDeviceInfo.sProduct, sDatetime, "Connected", m_usbDeviceInfo.sSerial,
                              m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                              m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
    } else if (USB_LIMIT_ZERO == m_sLimitModel) {
        m_disConnectFlag = disconnectDevice(m_usbDeviceInfo.sSysPath);
        if (m_disConnectFlag == 0) {
            Q_EMIT sendConnectionRecord(sBlockedUsbDeviceInfoStr);
            Q_EMIT requestShowBlockAllSysNotify();
            // clean
            cleanUsbConnectionLog();
            // updateNameInWhiteLst
            updateNameInWhiteLst(m_usbDeviceInfo.sProduct, m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
            // 写入记录
            QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
            writeConnectionRecord(m_usbDeviceInfo.sProduct, sDatetime, "Blocked", m_usbDeviceInfo.sSerial,
                                  m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                  m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
            return;
        } else {
            Q_EMIT sendConnectionRecord(sConnectedUsbDeviceInfoStr);
            // clean
            cleanUsbConnectionLog();
            // updateNameInWhiteLst
            updateNameInWhiteLst(m_usbDeviceInfo.sProduct, m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
            // 写入记录
            QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
            writeConnectionRecord(m_usbDeviceInfo.sProduct, sDatetime, "Connected", m_usbDeviceInfo.sSerial,
                                  m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                  m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
        }
    } else if (USB_LIMIT_PARTY == m_sLimitModel) {
        int nFlag = checkWhitelist(m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
        if (nFlag == 0) {
            Q_EMIT sendConnectionRecord(sConnectedUsbDeviceInfoStr);
            // clean
            cleanUsbConnectionLog();
            // updateNameInWhiteLst
            updateNameInWhiteLst(m_usbDeviceInfo.sProduct, m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
            // 写入记录
            QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
            writeConnectionRecord(m_usbDeviceInfo.sProduct, sDatetime, "Connected", m_usbDeviceInfo.sSerial,
                                  m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                  m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
        } else {
            m_disConnectFlag = disconnectDevice(m_usbDeviceInfo.sSysPath);
            if (m_disConnectFlag == 0) {
                Q_EMIT sendConnectionRecord(sBlockedUsbDeviceInfoStr);
                Q_EMIT requestShowOnlyWhiteListSysNotify();
                // clean
                cleanUsbConnectionLog();
                // updateNameInWhiteLst
                updateNameInWhiteLst(m_usbDeviceInfo.sLabel, m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
                // 写入记录
                QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
                writeConnectionRecord(m_usbDeviceInfo.sProduct, sDatetime, "Blocked", m_usbDeviceInfo.sSerial,
                                      m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                      m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
                return;
            } else {
                Q_EMIT sendConnectionRecord(sConnectedUsbDeviceInfoStr);
                // clean
                cleanUsbConnectionLog();
                // updateNameInWhiteLst
                updateNameInWhiteLst(m_usbDeviceInfo.sProduct, m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
                // 写入记录
                QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
                writeConnectionRecord(m_usbDeviceInfo.sProduct, sDatetime, "Connected", m_usbDeviceInfo.sSerial,
                                      m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                                      m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
            }
        }
    } else {
        Q_EMIT sendConnectionRecord(sConnectedUsbDeviceInfoStr);
        // clean
        cleanUsbConnectionLog();
        // updateNameInWhiteLst
        updateNameInWhiteLst(m_usbDeviceInfo.sProduct, m_usbDeviceInfo.sSerial, m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct);
        // 写入记录
        QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
        writeConnectionRecord(m_usbDeviceInfo.sProduct, sDatetime, "Connected", m_usbDeviceInfo.sSerial,
                              m_usbDeviceInfo.sIdVendor, m_usbDeviceInfo.sIdProduct,
                              m_usbDeviceInfo.sParentSize, m_usbDeviceInfo.sSize);
    }
}

QString UsbManageMonitor::getProcUserName(QString procName)
{
    QString procUser;

    PROCTAB* proc = openproc(
        PROC_FILLMEM |          // memory status: read information from /proc/#pid/statm
        PROC_FILLSTAT |         // cpu status: read information from /proc/#pid/stat
        PROC_FILLUSR            // user status: resolve user ids to names via /etc/passwd
        );
    static proc_t proc_info;
    memset(&proc_info, 0, sizeof(proc_t));

    while (readproc(proc, &proc_info) != nullptr) {
        QString cmdLine = getProcessCmdLine(proc_info.tid);
        if (cmdLine.contains(procName)) {
            procUser = proc_info.euser;
            return procUser;
        }
//        qDebug() << proc_info.tid << proc_info.cmd << proc_info.euser;
    }
    closeproc(proc);
    qDebug() << QString("Get %1's username failed!").arg(procName);
    return procUser;
}

QString UsbManageMonitor::getProcessCmdLine(pid_t pid)
{
    std::string temp;
    try {
        std::fstream fs;
        fs.open("/proc/"+std::to_string((long)pid)+"/cmdline", std::fstream::in);
        std::getline(fs,temp);
        // 先将数据同步到磁盘，然后再关闭文件，防止强制断电后损坏文件
        fs.sync();
        fs.close();
    } catch(std::ifstream::failure e) {
        return "FAILED TO READ PROC";
    }

    // change \0 to ' '
    std::replace(temp.begin(),temp.end(),'\0',' ');

    if (temp.size()<1) {
        return "";
    }

    QStringList retStrList = QString::fromStdString(temp).trimmed().split(" ");
    QString retStr;
    if (0 < retStrList.count()) retStr = retStrList.first();
    return retStr;
}



void UsbManageMonitor::updateNameInWhiteLst(QString sName, QString sSerial, QString sVendorID, QString sProductID)
{
    QList<UsbConnectionInfo> lastUsbWhiteList = m_whiteList;
    // 清空数据之前，先备份数据
    backupConnectionWhiteLst();
    clearConnectionWhiteLst();

    for (UsbConnectionInfo info : lastUsbWhiteList) {
        if (sSerial == info.sSerial && sVendorID == info.sVendorID && sProductID == info.sProductID) {
            info.sName = sName;
        }
        writeConnectionWhiteLst(info.sName, info.sDatetime, info.sStatus, info.sSerial, info.sVendorID ,info.sProductID,
                                info.sParentSize, info.sSize);
    }
    // 清除白名单备份数据
    clearBackupOfConnectionWhiteLst();
}

QByteArray UsbManageMonitor::encryptString(QString data, qint64 nOriginSize)
{
    QByteArray dataBa = data.toUtf8();
    int byteSize = dataBa.size();

    QByteArray encryptDataBa;
    encryptDataBa.resize(byteSize);
    // encrypt index = origin file size % CryptionBufferSize
    int cryptionBufferIndex = nOriginSize % CryptionBufferSize;
    for (int i = 0; i < byteSize; i++) {
        encryptDataBa[i] = dataBa.at(i) + CryptionBuffer[cryptionBufferIndex];
        cryptionBufferIndex++;
        if (CryptionBufferSize - 1 < cryptionBufferIndex) {
            cryptionBufferIndex = 0;
        }
    }

    return encryptDataBa;
}

QString UsbManageMonitor::decryptBinary(QByteArray encryptBin)
{
    int byteSize = encryptBin.size();

    QByteArray decryptDataBa;
    decryptDataBa.resize(byteSize);

    int cryptionBufferIndex = 0;
    for (int i = 0; i < byteSize; i++) {
        decryptDataBa[i] = encryptBin.at(i) - CryptionBuffer[cryptionBufferIndex];
        cryptionBufferIndex++;
        if (CryptionBufferSize - 1 < cryptionBufferIndex) {
            cryptionBufferIndex = 0;
        }
    }

    return QString::fromUtf8(decryptDataBa);
}
