#ifndef USBMANAGEMONITOR_H
#define USBMANAGEMONITOR_H

#include <QObject>
#include <QSettings>

struct UsbDeviceInfo {
    QString sProduct;           // 产品型号
    QString sSerial;            // 序列号
    QString sIdVendor;          // 生产商ID
    QString sIdProduct;         // 产品ID
    QString sInterfaceClass;    // 接口类型
    QString sBusnum;            // bus总线序号
    QString sDevNum;            // 设备序号
    QString sSysPath;           // /sys/ 下路径
    QString sDevPath;           // 设备device路径，与sSysPath一致
    QString sSubSystem;         // 子系统
    QString sDevType;           // 设备类型
    QString sDevNode;           // 设备节点
    QString sLabel;             // 分区标签
    QString sMountPoint;        // 挂载点
    QString sParentSize;        // 父设备大小
    QString sSize;              // 分区大小
};

struct BlkInfo {
    QString type;
    QString parentDevPath;
    QString devPath;
    QString mountPoint;
    QString canRemove;
};

//usb连接日志信息结构
struct UsbConnectionInfo {
    QString sName;
    QString sDatetime;
    QString sStatus;
    QString sId;
    QString sSerial;
    QString sVendorID;
    QString sProductID;
    QString sParentSize;        // 父设备大小
    QString sSize;              // 分区大小
};

class UsbManageMonitor : public QObject
{
    Q_OBJECT
public:
    explicit UsbManageMonitor(QObject *parent = nullptr);
    ~UsbManageMonitor();
    void init();
    void run();
    void quit();
    // 将待发送设备信息整合成字符串
    QString mergeDevSavingInfoToStr(QString sName, QString sStatus, QString sSerial, QString sVendorID,
                                    QString sProductID, QString sParentSize, QString sSize);

    void writeConnectionRecord(QString name, QString sDatetime, QString status,
                               QString serial, QString vendorID, QString productID,
                               QString sParentSize, QString sSize);
    // 备份原有数据（在每次清除连接记录之前必须调用）
    void backupUsbConnectionLog();
    // 清除备份数据（在写完所有数据后必须调用）
    void clearBackupOfUsbConnectionLog();
    void clearUsbConnectionLog();
    void cleanUsbConnectionLog();

    void writeConnectionWhiteLst(QString name, QString sDatetime, QString status,
                                 QString serial, QString vendorID, QString productID,
                                 QString sParentSize, QString sSize);
    // 备份白名单原有数据（在每次清除白名单之前必须调用）
    void backupConnectionWhiteLst();
    // 清除白名单备份数据（在写完所有白名单数据后必须调用）
    void clearBackupOfConnectionWhiteLst();
    void clearConnectionWhiteLst();
    void removeFromWhiteLst(QString sSerial, QString sVendorID, QString sProductID);

    void changeIsbSaveRecord(bool change);
    void selectLimitationModel(QString modelStr);
    inline bool getRecordSaveSwitch() {return m_isbSaveRecord;}
    inline QString getLimitModel() {return m_sLimitModel;}
    QString getUsbConnectionRecords();
    QString getWhiteList();

private:
    //读取配置数据
    void loadConfig();
    void readConnectionRecord();
    void readConnectionWhiteLst();
    //首次启动弹出当前可移动设备
    void disconnectAllDevices();
    // 首次启动按白名单对usb设备进行断开处理
    void preManage();
    // 判断是否处于授权管理激活审核模式
    bool isActivatorInVerifingModel();
    void monitorDevices(struct udev *udev);
    void processDevice(struct udev_device *dev);
    // 检查设备是否在白名单中，存在则返回0
    int checkWhitelist(QString sSerial, QString sIdVendor, QString sIdProduct);
    // 断开设备，成功返回0
    int disconnectDevice(struct udev_device *dev);
    int disconnectDevice(QString sysPath);
    // 获取设备分区名称
    QString getDevMountPoint(QString sDevNode);
    // 获取分区标签
    QString getDevLabel(QString sDevNode);
    // 获取设备容量大小
    QString getDevSize(QString sDevNode);
    // 获取设备文件系统类型
    QString getDevFsType(QString sDevNode);
    // 是否是mtp设备
    bool isbMtpDevice(QString sBusNum, QString sDevNum);

    // manage usb disk storage
    void manageDiskStorage();
    // manage usb mobile storage
    void manageMobileStorage();
    //获取指定进程所属的用户，用于获取用户目录地址
    QString getProcUserName(QString procName);
    QString getProcessCmdLine(pid_t pid);
    void updateNameInWhiteLst(QString sName, QString sSerial, QString sVendorID, QString sProductID);
    //文件解加密
    QByteArray encryptString(QString sData, qint64 nOriginSize = 0);
    QString decryptBinary(QByteArray encryptBin);
Q_SIGNALS:
    //usb设备接入的信号，入参bflag 为false,表示限制状态
    void checkUsbInsert(bool bFlag, QString sUsbInfo);
    void sendConnectionRecord(QString sInfos);
    //通知显示限制所有usb设备的系统通知
    void requestShowBlockAllSysNotify();
    //通知显示限制非白名单usb设备的系统通知
    void requestShowOnlyWhiteListSysNotify();
public slots:

private:
    struct udev *m_udev;
    //配置文件路径
    QString m_configPath;
    //初始配置
    QSettings *m_config;
    //运行状态
    int m_status; //1-运行 0-停止
    //是否保存记录标志
    bool m_isbSaveRecord;
    //limitation model
    QString m_sLimitModel;
    QList<UsbConnectionInfo> m_usbConnectionInfoLst;
    QList<UsbConnectionInfo> m_whiteList;

    //device whether has been managed,true - managed
    bool m_isbManagedDiskStorage;
    bool m_isbManagedMobileStorage;
    //断开设备标志 0-断开
    int m_disConnectFlag;
    UsbDeviceInfo m_usbDeviceInfo;
    QString m_usbDeviceInfoStr;

};

#endif // USBMANAGEMONITOR_H
