/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "antiviruswork.h"

// 安全中心配置目录
const QString SConfigDir = "/usr/bin/config/deepin-defender/";
// 本地病毒库版本文件
const QString SLocalVdbVerCfgFileName = "version";
// 本地病毒库默认版本
const QString SLocalVdbDefaultVer = "00.0.00.01";
// gsetting上次检测时间键
const QString SLastScanTimeKey = "last-scan-time";

AntiVirusWork::AntiVirusWork(QObject *parent)
    : QObject(parent)
    , m_AntivirusServer(nullptr)
    , m_DataInterFaceServer(nullptr)
    , m_gsetting(nullptr)
    , m_scanResultBuffer()
    , m_srvScanningStatus(DissectionState::Continue)
    , m_scanningStatus(ScanningStatus::None)
    , m_isbUsbDissection(false)
    , m_sUsbScanPath("")
    , m_sSrvVdbVer(SLocalVdbDefaultVer)
    , m_sLocalVdbVer(SLocalVdbDefaultVer)
    , m_vdbUpdating(false)
{
    // 病毒查杀服务dbus接口
    m_AntivirusServer = new AntivirusServer("com.deepin.defender.antiav",
                                            "/com/deepin/defender/antiav",
                                            QDBusConnection::systemBus(), this);
    //mxz 0910 由于注销不会杀死服务, work初始化 对扫描服务停止
    // bug https://pms.uniontech.com/zentao/bug-view-45945.html
    m_AntivirusServer->setDissection(2);
    m_AntivirusServer->SetIsbInVirusScanningPage(false);
    // 安全中心数据服务dbus接口
    m_DataInterFaceServer = new DataInterFaceServer("com.deepin.defender.datainterface",
                                                    "/com/deepin/defender/datainterface",
                                                    QDBusConnection::sessionBus(), this);
    // 安全中心gsetting
    m_gsetting = new QGSettings("com.deepin.dde.deepin-defender", QByteArray(), this);

    // 获得本地病毒库版本
    readLocalVdbVersion();
    // 初始化服务器病毒库版本
    m_sSrvVdbVer = m_sLocalVdbVer;
    // mxz 为了避免出现获取版本卡顿（病毒服务异常）导致页面卡顿　　先默认给一个值　线程在后台获取数据
    thread = new QThread();
    m_versionThread = new VdbVersionThread();
    m_versionThread->moveToThread(thread);
    connect(thread, &QThread::finished, thread, &QThread::deleteLater);
    // 连接请求获取服务器病毒库版本信号
    connect(this, &AntiVirusWork::requestGetVdbVersion, m_versionThread, &VdbVersionThread::getVdbVersion);
    // 连接发送的服务器的病毒库版本
    connect(m_versionThread, &VdbVersionThread::sendVdbVersion, this, &AntiVirusWork::recVdbVersion);
    thread->start();
    Q_EMIT requestGetVdbVersion();

    // 连接扫描结果信号
    connect(m_AntivirusServer, &AntivirusServer::onDissectionResult, this, &AntiVirusWork::onRecDissectionResult);
    // 连接扫描结束信号
    connect(m_AntivirusServer, &AntivirusServer::signal_doDissectionFinished, this, &AntiVirusWork::sendDissectionFinishedSignal);
    connect(m_AntivirusServer, &AntivirusServer::signal_doDissectionFinished, [ = ] {
        bool isScanning = m_AntivirusServer->isScanning();
        qDebug() << "isScanning======" << isScanning;
    });
    // 连接设置扫描状态结果信号
    connect(m_AntivirusServer, &AntivirusServer::onSetDissectionStateResult, this, &AntiVirusWork::sendSetDissectionStateResult);
    // 连接修复文件结果信号
    connect(m_AntivirusServer, &AntivirusServer::onRepairProblemFileResult, this, &AntiVirusWork::sendRepairProblemFileResult);
    // 连接修改信任文件结果信号
    connect(m_AntivirusServer, &AntivirusServer::onReposeProblemFileResult, this, &AntiVirusWork::sendChangeTrustFilesResult);
    // 连接获取信任文件信号
    connect(m_AntivirusServer, &AntivirusServer::signal_onQueryReposeFile, this, &AntiVirusWork::onRecTrustFiles);
    // 连接修改隔离文件结果信号
    connect(m_AntivirusServer, &AntivirusServer::onSepProblemFileResult, this, &AntiVirusWork::sendChangeIsolationFilesResult);
    // 连接获取隔离文件信号
    connect(m_AntivirusServer, &AntivirusServer::signal_onQuerySepFile, this, &AntiVirusWork::onRecIsolationFiles);
    // 连接获取更新结果信号
    connect(m_AntivirusServer, &AntivirusServer::signal_UpdateVersionResult, this, &AntiVirusWork::onUpdateVDbFinished);
    // 连接查杀未处理异常文件
    connect(m_AntivirusServer, &AntivirusServer::onRequstScanThreatsFile, this, &AntiVirusWork::scanThreatsFile);
}

AntiVirusWork::~AntiVirusWork()
{
    thread->quit();
}

void AntiVirusWork::setIsbInVirusScanningPage(bool isbScanning)
{
    m_AntivirusServer->SetIsbInVirusScanningPage(isbScanning);
}

void AntiVirusWork::setScanningStatus(AntiVirusWork::ScanningStatus status)
{
    qDebug() << "setScanningStatus: " << status;
    m_scanningStatus = status;
    if (ScanningStatus::Result == status) {
        // 如果界面扫描结束（且存在危险项）, 发送界面扫描结束信号
        Q_EMIT this->scanningFinished();
    } else if (ScanningStatus::ResultNoThreats == status) {
        // 如果界面扫描结束（不存在危险项）, 发送界面扫描结束信号
        Q_EMIT this->scanningFinished();
    } else if (ScanningStatus::Scanning == status) {
        Q_EMIT this->startScanning();
    }
}

void AntiVirusWork::doDissection(bool isbAll, QString sPaths)
{
    m_AntivirusServer->doDissection(isbAll, sPaths);
    requestSetDissectionStatus(DissectionState::Continue);
}

void AntiVirusWork::requestSetDissectionStatus(DissectionState status)
{
    int iStatus = 1;
    // 转换扫描状态为int型
    switch (status) {
    case DissectionState::Pause:
        iStatus = 0;
        break;
    case DissectionState::Continue:
        iStatus = 1;
        break;
    case DissectionState::Stop:
        iStatus = 2;
        break;
    }
    m_srvScanningStatus = status;
    m_AntivirusServer->setDissection(iStatus);
}

void AntiVirusWork::requestRepairProblemFile(QString sPath)
{
    qDebug() << "修复文件===" << sPath;
    m_AntivirusServer->repairProblemFile(sPath);
}

void AntiVirusWork::requestChangeTrustFiles(bool isbAdd, QString sPath)
{
    qDebug() << "信任文件===" << sPath << ",是否添加" << isbAdd;
    m_AntivirusServer->reposeProblemFile(isbAdd, sPath);
}

void AntiVirusWork::requestQueryTrustFiles()
{
    m_AntivirusServer->queryReposeFile();
}

void AntiVirusWork::requestChangeIsolationFile(bool isbAdd, QString sPath)
{
    qDebug() << "隔离文件===" << sPath << ",是否添加" << isbAdd;;
    m_AntivirusServer->sepProblemFile(isbAdd, sPath);
}

void AntiVirusWork::requestQueryIsolationFile()
{
    m_AntivirusServer->querySepFile();
}

// 获取上次扫描时间
QString AntiVirusWork::lastScanningTime()
{
    return m_gsetting->get(SLastScanTimeKey).toString();
}

// 设置上次扫描时间
void AntiVirusWork::setLastScanningTime(QString sTime)
{
    m_gsetting->set(SLastScanTimeKey, sTime);
}

// 以当前时间更新上次扫描时间
void AntiVirusWork::updateLastScanningTime()
{
    QDateTime date = QDateTime::currentDateTime();
    QString sLastScanningTime = date.toString("yyyy-MM-dd hh:mm:ss");
    // 设置上次扫描时间
    setLastScanningTime(sLastScanningTime);
}

void AntiVirusWork::requestUpdateVirusDb()
{
    m_AntivirusServer->updateVersion();
    // 设置正在更新状态位true
    m_vdbUpdating = true;
    // 通知病毒查杀首页更新页面数据
    Q_EMIT notifyUpdateAntiVirusHomeUI();
}

bool AntiVirusWork::isScanning()
{
    // 如果服务正在扫描 或 处于扫描界面，则返回true
    return m_AntivirusServer->isTrueScanning();
}

void AntiVirusWork::scanThreatsFile()
{
    Q_EMIT sendRequstScanThreatsFile();
}

void AntiVirusWork::setIsbUsbDissection(bool isbUsb)
{
    m_isbUsbDissection = isbUsb;
}

// 设置扫描开始
void AntiVirusWork::setScanStart()
{
    // 设置扫描开始
    m_AntivirusServer->setScanStart();
}

void AntiVirusWork::onRecDissectionResult(QString sPath, double dProgress, int iType, QString sDetailInfo)
{
    FileInfo file;
    file.path = sPath;
    file.type = Type(iType);
    file.detail = sDetailInfo;
    file.dProgress = dProgress;
    m_scanResultBuffer.append(file);

    //qDebug() << "sPath" << sPath << ", dProgress" << dProgress <<",jieshou==" << iType<<" zhuang type==" <<file.type;
    emit sendDissectionResult(sPath, dProgress, iType, sDetailInfo);
}

void AntiVirusWork::onRecTrustFiles(QString sInfos)
{
    QList<TrustFileInfo> trustFileInfos;
    //接受数据分割为一个一个信息
    QStringList firstCutStrList = sInfos.split("[*]");
    for (QString infoStr : firstCutStrList) {
        QStringList secondCutStrList = infoStr.split("*");
        if (2 > secondCutStrList.size())
            continue;
        TrustFileInfo info;
        info.sFilePath = secondCutStrList.at(0);
        info.sFileName = info.sFilePath.split("/").last();
        info.sAddTime = secondCutStrList.at(1);
        //添加到信任列表
        trustFileInfos.append(info);
    }
    qDebug() << "查询信任文件长度==" << trustFileInfos.size();
    Q_EMIT sendTrustFilesByList(trustFileInfos);
}

void AntiVirusWork::onRecIsolationFiles(QString sInfos)
{
    QList<TrustFileInfo> isolationFileInfos;
    //接受数据分割为一个一个信息
    QStringList firstCutStrList = sInfos.split("[*]");
    for (QString infoStr : firstCutStrList) {
        QStringList secondCutStrList = infoStr.split("*");
        if (2 > secondCutStrList.size())
            continue;
        TrustFileInfo info;
        info.sFilePath = secondCutStrList.at(0);
        info.sFileName = info.sFilePath.split("/").last();
        info.sAddTime = secondCutStrList.at(1);
        //添加到信任列表
        isolationFileInfos.append(info);
    }
    qDebug() << "查询隔离文件长度==" << isolationFileInfos.size();
    Q_EMIT sendIsolationFilesByList(isolationFileInfos);
}

// 接收来自线程的病毒库版本
void AntiVirusWork::recVdbVersion(QString sServerVdbVer)
{
    qDebug() << "recieved vdb version from thread: " << sServerVdbVer;

    // 若当前版本与服务器病毒库版本相同，则不予处理
    if (m_sLocalVdbVer == sServerVdbVer) {
        return;
    }
    // 格式校验
    if (4 != sServerVdbVer.split(".").count()) {
        return;
    }

    // 若当前版本与服务器病毒库版本不同，则存储服务器病毒库版本号
    m_sSrvVdbVer = sServerVdbVer;
    // 通知打开病毒库更新对话框
    Q_EMIT notifyOpenVdbUpdateDialog();
    // 通知更新病毒查杀首页数据
    Q_EMIT notifyUpdateAntiVirusHomeUI();
}

void AntiVirusWork::onUpdateVDbFinished(bool isbSus)
{
    // 设置是否正在更新本地病毒库标志位FALSE
    m_vdbUpdating = false;

    if (isbSus) {
        // 若更新成功，则读取更新后的本地病毒库版本
        readLocalVdbVersion();
    } else {
        // 若更新失败，则通知打开更新失败弹窗
        Q_EMIT notifyOpenVdbUpdateFailedDialog();
    }
    // 通知更新病毒查杀首页数据
    Q_EMIT notifyUpdateAntiVirusHomeUI();
}

// 获得本地病毒库版本
void AntiVirusWork::readLocalVdbVersion()
{
    //文件名
    QString m_fileName = SConfigDir + SLocalVdbVerCfgFileName;
    QFile f(m_fileName);
    if (!f.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << SConfigDir + SLocalVdbVerCfgFileName << "open failed!";
        m_sLocalVdbVer = SLocalVdbDefaultVer;
        return;
    }
    //输入流
    QTextStream txtInput(&f);
    QString sLocalVdbVer = txtInput.readLine(0);

    // 格式校验
    if (4 == sLocalVdbVer.split(".").count()) {
        m_sLocalVdbVer = sLocalVdbVer;
    } else {
        m_sLocalVdbVer = SLocalVdbDefaultVer;
    }

    qDebug() << "local virus databas version:" << m_sLocalVdbVer;
    f.close();
}
