/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "com_deepin_defender_antiav.h"
#include "com_deepin_defender_datainterface.h"
#include "cominfo.h"

#include "vdbversionthread.h"

#include <QObject>
#include <QGSettings>

//dbus接口 服务名称
using AntivirusServer = com::deepin::defender::antiav;
using DataInterFaceServer = com::deepin::defender::datainterface;

// 安全中心配置目录
extern const QString SConfigDir;
// 本地病毒库版本文件
extern const QString SLocalVdbVerCfgFileName;
// 本地病毒库默认版本
extern const QString SLocalVdbDefaultVer;
// gsetting上次检测时间键
extern const QString SLastScanTimeKey;

///
/// \brief The AntiVirusWork class
/// 病毒查杀数据处理类
class AntiVirusWork: public QObject
{
    Q_OBJECT
public:
    // 扫描状态枚举 - 没有扫描 扫描中 结果展示中  结果页面但是无异常
    enum ScanningStatus {None, Scanning, Result, ResultNoThreats};
    explicit AntiVirusWork(QObject *parent);
    virtual ~AntiVirusWork();

    void setIsbInVirusScanningPage(bool isbScanning);
    // 设置扫描状态
    void setScanningStatus(AntiVirusWork::ScanningStatus status);
    // 获取扫描状态
    inline ScanningStatus scanningStatus() {return m_scanningStatus;}

    // 扫描 （全盘true/自定义false）
    void doDissection(bool isbAll, QString sPaths);
    // 获取检测结果缓冲区列表
    QVector<FileInfo> &getScanResultList() {return m_scanResultBuffer;}
    // 设置扫描状态 （暂停/继续/终止）
    void requestSetDissectionStatus(DissectionState status);
    // 修复问题文件
    void requestRepairProblemFile(QString sPath);
    // 信任问题文件 (添加/移除)
    void requestChangeTrustFiles(bool isbAdd, QString sPath);
    // 查询信任文件
    void requestQueryTrustFiles();
    // 隔离问题文件 (添加/移除)
    void requestChangeIsolationFile(bool isbAdd, QString sPath);
    // 查询隔离文件
    void requestQueryIsolationFile();
    // 获取上次扫描时间
    QString lastScanningTime();
    // 设置上次扫描时间
    void setLastScanningTime(QString sTime);
    // 以当前时间更新上次扫描时间
    void updateLastScanningTime();

    //版本更新
    void requestUpdateVirusDb();
    // 是否在扫描中
    bool isScanning();
    // 本地病毒库版本
    inline QString localVdbVersion() {return m_sLocalVdbVer;}
    // 服务器病毒库版本
    inline QString srvVdbVersion() {return m_sSrvVdbVer;}
    // 是否正在更新病毒库
    inline bool vdbUpdating() {return m_vdbUpdating;}

    // 查杀为处理异常文件
    void scanThreatsFile();

    // 设置是否是usb储存设备查杀
    void setIsbUsbDissection(bool isbUsb);

    // 设置扫描开始
    void setScanStart();

signals:
    // 扫描返回槽
    void sendDissectionResult(QString sPath, double dProgress, int iType, QString sDetailInfo);
    void sendDissectionFinishedSignal();
    // 发送设置检测状态结果
    void sendSetDissectionStateResult(QString sPath, QString sMessage);
    // 修复问题文件返回槽
    void sendRepairProblemFileResult(QString sPath, QString sMessage);
    // 信任问题文件返回槽
    void sendChangeTrustFilesResult(QString sPath, QString sMessage);
    void sendTrustFilesByList(QList<TrustFileInfo> sInfos);
    // 隔离问题文件返回槽
    void sendChangeIsolationFilesResult(QString sPath, QString sMessage);
    void sendIsolationFilesByList(QList<TrustFileInfo> sInfos);

    void sendQueryVersion(QString version);   //版本信号
    void sendUpdateVersionResult(bool flag);    //版本更新结果

    void sendRequstScanThreatsFile();   // 查杀未处理异常文件
    void requestGetVdbVersion();
    // 界面开始扫描信号
    void startScanning();
    // 界面扫描结束信号
    void scanningFinished();
    // 通知打开病毒库更新对话框
    void notifyOpenVdbUpdateDialog();
    // 通知打开病毒库更新失败对话框
    void notifyOpenVdbUpdateFailedDialog();
    // 通知更新病毒查杀首页数据
    void notifyUpdateAntiVirusHomeUI();
public slots:
    void onRecDissectionResult(QString sPath, double dProgress, int iType, QString sDetailInfo);
    void onRecTrustFiles(QString sInfos);
    void onRecIsolationFiles(QString sInfos);
    // 接收来自线程的病毒库版本
    void recVdbVersion(QString sServerVdbVer);
    // 当病毒库更新完成
    void onUpdateVDbFinished(bool isbSus);
private:
    // 获得本地病毒库版本
    void readLocalVdbVersion();

private:
    AntivirusServer         *m_AntivirusServer = nullptr;         // 病毒库dbus 接口
    DataInterFaceServer     *m_DataInterFaceServer = nullptr;     // 联网管控接口
    QGSettings              *m_gsetting;
    QVector<FileInfo>       m_scanResultBuffer;                   // 检测结果缓冲区列表
    DissectionState         m_srvScanningStatus;                  // 服务扫描状态
    ScanningStatus          m_scanningStatus;                     // 界面扫描状态
    bool                    m_isbUsbDissection;                   // 是否是usb储存设备查杀
    QString                 m_sUsbScanPath;
    // 当前病毒库版本
    QString m_sSrvVdbVer;
    QString m_sLocalVdbVer;
    bool                    m_vdbUpdating;                     // 是否正在更新病毒库
    QThread                 *thread = nullptr ;
    VdbVersionThread        *m_versionThread = nullptr;           // 获得版本线程
};
