/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "networkprotectionmodel.h"

#include <QGSettings>
#include <QVariant>
#include <QCollator>
#include <QDebug>

static const QString NP_SHARE_OPTION = "np-share-option";
static const QString NP_NETWORK_ACCESS = "np-network-access";
static const QString NP_REMOTE_ACCESSED = "np-remote-accessed";
static const QString NP_APP_ACCESS_OPTIONS = "np-app-access-options";
static const QString NP_APP_ASK_APPS = "np-app-ask-apps";
static const QString NP_APP_ALLOW_APPS = "np-app-allow-apps";
static const QString NP_APP_DISABLE_APPS = "np-app-disable-apps";

NetworkProtectionModel::NetworkProtectionModel(QObject *parent)
    : QObject(parent)
    , m_gs(new QGSettings("com.deepin.dde.deepin-defender", QByteArray(), this))
{
    m_shareValue = static_cast<NetworkShareOption>(m_gs->get(NP_SHARE_OPTION).toInt());
    m_accessNetwork = static_cast<NetworkAccessOption>(m_gs->get(NP_NETWORK_ACCESS).toInt());
    m_remoteAccessed = static_cast<NetworkAccessOption>(m_gs->get(NP_REMOTE_ACCESSED).toInt());

    initAskApp();
    initAllowApp();
}

void NetworkProtectionModel::reset()
{
    m_gs->reset(NP_SHARE_OPTION);
    m_gs->reset(NP_NETWORK_ACCESS);
    m_gs->reset(NP_REMOTE_ACCESSED);
    Q_EMIT shareOptionChanged(INVISIBLE);
    Q_EMIT networkAccessOptionChanged(DEFAULT_ASK);
    Q_EMIT remoteAccessedOptionChanged(DEFAULT_ASK);
}

NetworkProtectionModel::NetworkShareOption NetworkProtectionModel::getShareOption()
{
    return m_shareValue;
}

void NetworkProtectionModel::setShareOption(NetworkProtectionModel::NetworkShareOption opt)
{
    if (m_shareValue != opt) {
        m_shareValue = opt;
        m_gs->set(NP_SHARE_OPTION, opt);
        Q_EMIT shareOptionChanged(opt);
    }
}

NetworkProtectionModel::NetworkAccessOption NetworkProtectionModel::getNetworkAccessOption()
{
    return m_accessNetwork;
}

QString NetworkProtectionModel::getNetworkAccessOptionName()
{
    QStringList list;
    list << tr("Ask") << tr("Disable") << tr("Allow");
    return list[m_accessNetwork];
}

void NetworkProtectionModel::setNetworkAccessOption(NetworkProtectionModel::NetworkAccessOption opt)
{
    if (m_accessNetwork != opt) {
        m_accessNetwork = opt;
        m_gs->set(NP_NETWORK_ACCESS, opt);
        Q_EMIT networkAccessOptionChanged(opt);
    }
}

NetworkProtectionModel::NetworkAccessOption NetworkProtectionModel::getRemoteAccessedOption()
{
    return m_remoteAccessed;
}

void NetworkProtectionModel::setRemoteAccessedOption(NetworkProtectionModel::NetworkAccessOption opt)
{
    if (m_remoteAccessed != opt) {
        m_remoteAccessed = opt;
        m_gs->set(NP_REMOTE_ACCESSED, opt);
        Q_EMIT remoteAccessedOptionChanged(opt);
    }
}

QList<NetworkProtectionModel::AppNetworkAccessStruct> NetworkProtectionModel::getRunningProcessOption()
{
    QList<NetworkProtectionModel::AppNetworkAccessStruct> retList;
    QString optStr = m_gs->get(NP_APP_ACCESS_OPTIONS).toString();

    QStringList list = optStr.split(":");

    for (QString app : list) {
        QStringList item = app.split(",");
        if (item.isEmpty() || item.length() != 5)
            continue;
        NetworkProtectionModel::AppNetworkAccessStruct opt;
        opt.appName = item[0].trimmed();
        opt.icon = item[1].trimmed();
        opt.path = item[2].trimmed();
        opt.type = static_cast<NetworkProtectionModel::AppType>(item[3].trimmed().toInt());
        opt.option = static_cast<NetworkProtectionModel::AppsAccessOption>(item[4].trimmed().toInt());
        retList << opt;
    }

    m_appNetworkAccessOptions.clear();
    m_appNetworkAccessOptions = retList;

    return m_appNetworkAccessOptions;
}

void NetworkProtectionModel::setRunningProcessOption(QList<NetworkProtectionModel::AppNetworkAccessStruct> list)
{
    if (m_appNetworkAccessOptions != list) {
        QString str;
        for (NetworkProtectionModel::AppNetworkAccessStruct appItem : list) {
            str += QString("%1,%2,%3,%4,%5:").arg(appItem.appName).arg(appItem.icon)
                    .arg(appItem.path).arg(appItem.type).arg(appItem.option);
        }

        if (str.size() > 1)
            str = str.left(str.size() - 1);

        m_gs->set(NP_APP_ACCESS_OPTIONS, str);

        m_appNetworkAccessOptions = list;
        qDebug() << "update app struct Q_EMIT appAccessOptionsChanged ";
        Q_EMIT appAccessOptionsChanged(list);
    }
}

QList<NetworkProtectionModel::AppNetworkAccessStruct> NetworkProtectionModel::getSortRunningProcessOption(int order, int column)
{
    QList<NetworkProtectionModel::AppNetworkAccessStruct> retList;
    QString optStr = m_gs->get(NP_APP_ACCESS_OPTIONS).toString();

    QStringList list = optStr.split(":");
    QLocale loc(QLocale::Chinese, QLocale::China);
    QCollator qco(loc);

    qSort(list.begin(), list.end(), qco);

    for (QString app : list) {
        QStringList item = app.split(",");
        if (item.isEmpty() || item.length() != 5)
            continue;
        NetworkProtectionModel::AppNetworkAccessStruct opt;
        opt.appName = item[0].trimmed();
        opt.icon = item[1].trimmed();
        opt.path = item[2].trimmed();
        opt.type = static_cast<NetworkProtectionModel::AppType>(item[3].trimmed().toInt());
        opt.option = static_cast<NetworkProtectionModel::AppsAccessOption>(item[4].trimmed().toInt());
        if (order == 0) {
            retList << opt;
        } else {
            retList.insert(0, opt);
        }
    }


    if (column == 1) {
        if (order == 0) {
            qSort(retList.begin(), retList.end(),
                  [](const AppNetworkAccessStruct &a, const AppNetworkAccessStruct &b) {
                return a.option < b.option;
            });
        } else {
            qSort(retList.begin(), retList.end(),
                  [](const AppNetworkAccessStruct &a, const AppNetworkAccessStruct &b) {
                return a.option > b.option;
            });
        }
    }

    m_appNetworkAccessOptions.clear();
    m_appNetworkAccessOptions = retList;

    return m_appNetworkAccessOptions;
}

void NetworkProtectionModel::setAppAskOption(const QStringList &appNamePath, NetworkProtectionModel::AppAskOption askOpt)
{
    QString outStr;

    switch (askOpt) {
    case AllowAlways: {
        //change app settings
        QList<NetworkProtectionModel::AppNetworkAccessStruct> apps = getRunningProcessOption();
        int index = getAppNetworkIndexbyName(appNamePath);
        setAppNetworkOption(index, ItemAllow);
        return;
    };
    case AllowOnce:{
//        if (!m_appAskOptions.contains(appName)) {
//            m_appAskOptions << appName;
//            QString outStr= m_appAskOptions.length() > 1 ? m_appAskOptions.join(":") : appName;
//            m_gs->set(NP_APP_ASK_APPS, outStr);
//        }
        break;
    }
    case Diallow:
        break;
    }
}

//
bool NetworkProtectionModel::getAppAskOption(const QString &appName)
{
    return (m_appAskOptions.contains(appName));
}

NetworkProtectionModel::AppsAccessOption NetworkProtectionModel::getAppNetworkAccessOption(const QString &appName)
{
    if (appName.isEmpty())
        return ItemAllow;

    for (AppNetworkAccessStruct appStruct : m_appNetworkAccessOptions) {
        if (appStruct.path.contains(appName)) {
            if (appStruct.option != ItemDefault) {
                return appStruct.option;
            } else {
                // check network opt
                switch (m_accessNetwork) {
                case DEFAULT_ASK:
                    return ItemAsk;
                case DEFAULT_ALLOW:
                    return ItemAllow;
                case DEFAULT_DISALLOW:
                    return ItemDisallow;
                }
            }
        }
    }

    // not found will allow the access request
    qDebug() << " not found, using default for app " << appName;
    return ItemAllow;
    switch (m_accessNetwork) {
    case DEFAULT_ASK:
        return ItemAsk;
    case DEFAULT_ALLOW:
        return ItemAllow;
    case DEFAULT_DISALLOW:
        return ItemDisallow;
    }
    return ItemAllow;
}

int NetworkProtectionModel::getAppNetworkIndexbyName(const QStringList &appNamePath)
{
    if (appNamePath.isEmpty() || (appNamePath.size() < 1))
        return -1;

    for (AppNetworkAccessStruct appStruct : m_appNetworkAccessOptions) {
        if ((appStruct.appName == appNamePath[0]))
            return m_appNetworkAccessOptions.indexOf(appStruct);
        if (appNamePath.size() == 2 && (appStruct.path == appNamePath[1]))
            return m_appNetworkAccessOptions.indexOf(appStruct);
    }
    return -1;
}

void NetworkProtectionModel::setAppNetworkOption(int index, NetworkProtectionModel::AppsAccessOption option)
{
    if (index < 0)
        return;

    QList<NetworkProtectionModel::AppNetworkAccessStruct> apps = getRunningProcessOption();
    NetworkProtectionModel::AppNetworkAccessStruct item = apps[index];
    NetworkProtectionModel::AppsAccessOption oldOpt = item.option;
    if (oldOpt == option)
        return;

    if (option == ItemAllow && m_appAskOptions.contains(item.appName)) {
        m_appAskOptions.removeOne(item.appName);
        QString outStr = m_appAskOptions.join(":");
        m_gs->set(NP_APP_ASK_APPS, outStr);
    }

    item.option = option;
    apps.replace(index, item);

    setRunningProcessOption(apps);
}

QString NetworkProtectionModel::foundAppPath(const QString &path)
{
    for (AppNetworkAccessStruct appStruct : m_appNetworkAccessOptions) {
        if (appStruct.path.contains(path, Qt::CaseInsensitive)) {
            return appStruct.appName;
        }
    }

    return "";
}

void NetworkProtectionModel::setAllowApp(const QString &appName, bool allow)
{
    if (allow) {
        if (!m_appAllowApps.contains(appName)) {
            m_appAllowApps << appName;
            QString outStr = m_appAllowApps.length() > 1 ? m_appAllowApps.join(":") : appName;
            m_gs->set(NP_APP_ALLOW_APPS, outStr);
        }
    } else {
        if (m_appAllowApps.contains(appName)) {
            m_appAllowApps.removeOne(appName);
            QString outStr = m_appAllowApps.length() > 1 ? m_appAllowApps.join(":") : appName;
            m_gs->set(NP_APP_ALLOW_APPS, outStr);
        }
    }
}

void NetworkProtectionModel::initAskApp()
{
    QString apps = m_gs->get(NP_APP_ASK_APPS).toString();
    m_appAskOptions = apps.split(":");
}

void NetworkProtectionModel::initAllowApp()
{
    QString allowApps = m_gs->get(NP_APP_ALLOW_APPS).toString();
    m_appAllowApps = allowApps.split(":");
}
