/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "networkprotectionworker.h"
#include "networkprotectionmodel.h"
#include "networkprotectionwidget.h"
#include "trafficdetailwidget.h"

#include <QFile>
#include <QProcess>
#include <QLabel>
#include <QMessageBox>
#include <QDateTime>
#include <QCoreApplication>
#include <QDebug>

NetworkProtectionWorker::NetworkProtectionWorker(NetworkProtectionModel *model, QObject *parent)
    : QObject(parent)
    , m_model(model)
    , m_launcherInter(new Launcherd("com.deepin.dde.daemon.Launcher", "/com/deepin/dde/daemon/Launcher", QDBusConnection::sessionBus(), this))
    , m_monitornetflow(new MonitorNetFlow("com.deepin.defender.MonitorNetFlow", "/com/deepin/defender/MonitorNetFlow", QDBusConnection::systemBus(), this))
    , m_DataInterFaceServer(new DataInterFaceServer("com.deepin.defender.datainterface", "/com/deepin/defender/datainterface", QDBusConnection::sessionBus(), this))
    , m_onDispFlowRange(FlowRangeStyle::None)
{
    connect(this, &NetworkProtectionWorker::notifyUpdateLauncherList, this, &NetworkProtectionWorker::getRunningProcessOption);

    onUpdateLauncherList();

    // 通知停止获取所有应用流量数据
    void notifyDontGetFlowData();

    //链接trafficdetails服务发送的所有应用流量数据信号
    registerDefenderProcInfoMetaType();
    registerDefenderProcInfoListMetaType();
    connect(m_DataInterFaceServer, &DataInterFaceServer::SendAllAppFlowList, this, &NetworkProtectionWorker::sendAllAppFlowList);

    //链接trafficdetails服务发送的应用流量数据信号
    connect(m_DataInterFaceServer, &DataInterFaceServer::SendAppFlowList, this, [ = ](DefenderProcInfoList list) {
        Q_EMIT sendAppFlowList(list);
        qDebug() << "SendAppFlowList" << list.size(); ///////////////////////////////////=============================
    });
}

void NetworkProtectionWorker::reset()
{
    m_model->reset();
}

QList<NetworkProtectionModel::AppNetworkAccessStruct> NetworkProtectionWorker::getRunningProcessOption()
{
    QList<NetworkProtectionWorker::AppIcon> launcherAppList = getAllApps();

    QList<NetworkProtectionModel::AppNetworkAccessStruct> modelAppList = m_model->getRunningProcessOption();

    QList<NetworkProtectionModel::AppNetworkAccessStruct> list;

    for (int i = 0; i < launcherAppList.size(); ++i) {
        NetworkProtectionModel::AppNetworkAccessStruct item = {
            launcherAppList[i].appName,
            launcherAppList[i].icon,
            launcherAppList[i].path,
            launcherAppList[i].type,
            NetworkProtectionModel::ItemDefault
        };

        if (item.type == NetworkProtectionModel::SystemApp)
            item.option = NetworkProtectionModel::ItemAllow;

        for (int j = 0; j < modelAppList.size(); ++j) {
            if (item.path == modelAppList[j].path) {
                item.option = modelAppList[j].option;
                break;
            }
        }
        list << item;
    }

    m_model->setRunningProcessOption(list);
    return list;
}

void NetworkProtectionWorker::setRunningProcessOption(QList<NetworkProtectionModel::AppNetworkAccessStruct> list)
{
    m_model->setRunningProcessOption(list);
}

QList<NetworkProtectionWorker::AppIcon> NetworkProtectionWorker::getAllApps()
{
    QDBusPendingReply<LauncherItemInfoList> reply = getLauncherList();
    reply.waitForFinished();
    if (reply.isError()) {
        qWarning() << "data is empty, quit!!";
        qWarning() << reply.error();
        qApp->quit();
    }

    const LauncherItemInfoList &datas = reply.value();
    QList<AppIcon> apps;

    for (const auto &it : datas) {
        AppIcon app;
        app.appName = it.Name;
        app.icon = it.Icon;
        app.path = it.Path;
        app.type = getAppType(it.Path);
        apps << app;
    }

    //add ping for test
//    AppIcon app;
//    app.appName = "ping";
//    app.type = NetworkProtectionModel::SystemApp;
//    app.path = "ping";
//    apps << app;

    return apps;
}

void NetworkProtectionWorker::setRunningProcessOption(int index, NetworkProtectionModel::AppsAccessOption option)
{
    m_model->setAppNetworkOption(index, option);
}

NetworkProtectionModel::AppType NetworkProtectionWorker::getAppType(const QString &path)
{
    NetworkProtectionModel::AppType type = NetworkProtectionModel::SystemApp;
    QFile fileInfo(path);
    if (QFileInfo(path).isSymLink()) {
        type = NetworkProtectionModel::ThirdPartyApp;
    } else if (fileInfo.open(QIODevice::ReadOnly | QIODevice::Text)) {
        QTextStream in(&fileInfo);
        in.setCodec("UTF-8");
        QString sAllInfo = in.readAll();
        int nIndex = sAllInfo.indexOf("Exec");
        QString sOptInfo = sAllInfo.mid(nIndex, 10);
        if (sOptInfo.contains("opt"))
            type = NetworkProtectionModel::ThirdPartyApp;
    }

    return type;
}

QString NetworkProtectionWorker::getAppTypeName(NetworkProtectionModel::AppType type)
{
    switch (type) {
    case NetworkProtectionModel::SystemApp:
        return (tr("System"));
    case NetworkProtectionModel::ThirdPartyApp:
        return (tr("Third-party"));
    }
    return "";
}

void NetworkProtectionWorker::showNetworkAccessSettings(const QStringList &appNamePath)
{
    Q_EMIT requestShowNetworkSettings(appNamePath);
}

QDBusPendingReply<LauncherItemInfoList> NetworkProtectionWorker::getLauncherList()
{
    return m_launcherInter->GetAllItemInfos();
}

//有应用下载/卸载
void NetworkProtectionWorker::onUpdateLauncherList()
{
    allApp app;
    m_allapp.clear();
    QDBusPendingReply<LauncherItemInfoList> m_reply;
    m_reply = m_launcherInter->GetAllItemInfos();
    m_reply.waitForFinished();
    if (m_reply.isError()) {
        qWarning() << "data is empty, quit!!";
        qWarning() << m_reply.error();
        qApp->quit();
    }
    const LauncherItemInfoList &datas = m_reply.value();
    for (auto at : datas) {
        app.Name = at.Name;
        app.Icon = at.Icon;
        app.ID = at.ID;
        app.Path = at.Path;
        m_allapp << app;
    }
//添加https到应用
    app.Name = "https";
    app.ID = "https";
    app.Icon = ":/imageresource/dcc_application_executable.svg";
    app.Path = "https";
    m_allapp << app;
    Q_EMIT notifyUpdate();
}




DefenderProcInfoList NetworkProtectionWorker::getProcInfoList()
{
    // 从启动器dbus服务中获取所有应用信息
    QDBusPendingReply<DefenderProcInfoList> reply = m_DataInterFaceServer->GetAllProcInfos();
    reply.waitForFinished();
    if (reply.isError()) {
        qDebug() << "data is empty, quit!!";
        qDebug() << reply.error();
    }
    return  reply.value();
}

void NetworkProtectionWorker::notifyGetYesterdayFlowData()
{
    m_onDispFlowRange = FlowRangeStyle::Yesterday;
    m_DataInterFaceServer->notifyGetYesterdayFlowData();
}

void NetworkProtectionWorker::notifyGetTodayFlowData()
{
    m_onDispFlowRange = FlowRangeStyle::Today;
    m_DataInterFaceServer->notifyGetTodayFlowData();
}

void NetworkProtectionWorker::notifyGetLastMonthFlowData()
{
    m_onDispFlowRange = FlowRangeStyle::LastMonth;
    m_DataInterFaceServer->notifyGetLastMonthFlowData();
}

void NetworkProtectionWorker::notifyGetThisMonthFlowData()
{
    m_onDispFlowRange = FlowRangeStyle::ThisMonth;
    m_DataInterFaceServer->notifyGetThisMonthFlowData();
}

// 通知停止获取所有应用流量数据
void NetworkProtectionWorker::notifyDontGetFlowData()
{
    m_onDispFlowRange = FlowRangeStyle::None;
    m_DataInterFaceServer->NotifyDontGetFlowData();
}

void NetworkProtectionWorker::notifyGetAppYesterdayFlowData(QString sExecPath)
{
    Q_EMIT m_DataInterFaceServer->notifyGetAppYesterdayFlowData(sExecPath);
}

void NetworkProtectionWorker::notifyGetAppTodayFlowData(QString sExecPath)
{
    Q_EMIT m_DataInterFaceServer->notifyGetAppTodayFlowData(sExecPath);
}

void NetworkProtectionWorker::notifyGetAppLastMonthFlowData(QString sExecPath)
{
    Q_EMIT m_DataInterFaceServer->notifyGetAppLastMonthFlowData(sExecPath);
}

void NetworkProtectionWorker::notifyGetAppThisMonthFlowData(QString sExecPath)
{
    Q_EMIT m_DataInterFaceServer->notifyGetAppThisMonthFlowData(sExecPath);
}
