/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "screensecuritywidget.h"

#include <DLabel>

#include <QVBoxLayout>

ScreenSecurityWidget::ScreenSecurityWidget(SystemSafetyModel *model, SystemSafetyWork *work
                                           , QWidget *parent)
    : QWidget(parent)
    , m_model(model)
    , m_work(work)
    , m_linePowerLockSliderItem(nullptr)
    , m_batteryLockSliderItem(nullptr)
{
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setAlignment(Qt::AlignLeft);
    mainLayout->setSpacing(10);
    setLayout(mainLayout);

    QStringList annos;
    annos << "1m" << "5m" << "10m" << "15m" << "30m" << "1h" << tr("Never");

    //使用电源时自动锁屏时间设置滚动条
    m_linePowerLockSliderItem = new TitledSliderItem(tr("Lock screen after (Plugged In)"));
    m_linePowerLockSliderItem->setParent(this);
    m_linePowerLockSliderItem->slider()->setType(DCCSlider::Vernier);
    m_linePowerLockSliderItem->slider()->setRange(1, 7);
    m_linePowerLockSliderItem->slider()->setTickPosition(QSlider::TicksBelow);
    m_linePowerLockSliderItem->slider()->setTickInterval(1);
    m_linePowerLockSliderItem->slider()->setPageStep(1);
    m_linePowerLockSliderItem->setAnnotations(annos);
    mainLayout->addWidget(m_linePowerLockSliderItem);
    //设置初始值
    int sliderValue = lockDelayValueToSliderValue(m_work->getLinePowerLockDelay());
    m_linePowerLockSliderItem->slider()->setValue(sliderValue);
    m_linePowerLockSliderItem->setValueLiteral(sliderValueToLiteralString(sliderValue));

    //使用电池时自动锁屏时间设置滚动条
    m_batteryLockSliderItem = new TitledSliderItem(tr("Lock screen after (On Battery)"));
    m_batteryLockSliderItem->setParent(this);
    m_batteryLockSliderItem->slider()->setType(DCCSlider::Vernier);
    m_batteryLockSliderItem->slider()->setRange(1, 7);
    m_batteryLockSliderItem->slider()->setTickPosition(QSlider::TicksBelow);
    m_batteryLockSliderItem->slider()->setTickInterval(1);
    m_batteryLockSliderItem->slider()->setPageStep(1);
    m_batteryLockSliderItem->setAnnotations(annos);
    mainLayout->addWidget(m_batteryLockSliderItem);
    //设置初始值
    sliderValue = lockDelayValueToSliderValue(m_work->getBatteryLockDelay());
    m_batteryLockSliderItem->slider()->setValue(sliderValue);
    m_batteryLockSliderItem->setValueLiteral(sliderValueToLiteralString(sliderValue));

    //判断是否有电池
    if (!m_work->haveBattery()) {
        m_batteryLockSliderItem->hide();
        m_linePowerLockSliderItem->setTitle(tr("Lock screen after"));
    }

    //设置锁屏壁纸项
    TitleButtonItem *lockItem = new TitleButtonItem;
    lockItem->setTitle(tr("Lockscreen wallpaper"));
    lockItem->setButtonText(tr("Change"));
    mainLayout->addWidget(lockItem);

    //更多设置
    DLabel *otherSettingLabel = new DLabel(tr("More Settings"));
    mainLayout->addWidget(otherSettingLabel);
    //设置自动息屏时间项
    TitleButtonItem *closeMonitorSettingItem = new TitleButtonItem;
    closeMonitorSettingItem->setTitle(tr("Turn off monitor"));
    closeMonitorSettingItem->setButtonText(tr("Settings"));
    mainLayout->addWidget(closeMonitorSettingItem);
    //设置屏幕保护
    TitleButtonItem *lockSaverItem = new TitleButtonItem;
    lockSaverItem->setTitle(tr("Screensaver"));
    lockSaverItem->setButtonText(tr("Settings"));
    mainLayout->addWidget(lockSaverItem);

    mainLayout->addStretch(1);

    //connect
    //获取使用电源时的锁屏时间
    connect(m_work, &SystemSafetyWork::linePowerLockDelayChanged, this, [=](int value) {
        int sliderValue = lockDelayValueToSliderValue(value);
        m_linePowerLockSliderItem->slider()->setValue(sliderValue);
        m_linePowerLockSliderItem->setValueLiteral(sliderValueToLiteralString(sliderValue));
    });
    //链接，使用电源时的锁屏时间滚动条数值改变时
    connect(m_linePowerLockSliderItem->slider(), &DCCSlider::valueChanged,
            this, &ScreenSecurityWidget::changeLinePowerLockDelay);

    //获取使用电池时的锁屏时间
    connect(m_work, &SystemSafetyWork::batteryLockDelayChanged, this, [=](int value) {
        int sliderValue = lockDelayValueToSliderValue(value);
        m_batteryLockSliderItem->slider()->setValue(sliderValue);
        m_batteryLockSliderItem->setValueLiteral(sliderValueToLiteralString(sliderValue));
    });
    //链接，使用电池时的锁屏时间滚动条数值改变时
    connect(m_batteryLockSliderItem->slider(), &DCCSlider::valueChanged,
            this, &ScreenSecurityWidget::changeBatteryLockDelay);

    //链接按钮
    connect(lockItem->button(), &QPushButton::clicked, [=]() {
        m_work->openLockBackgroundSetting();
    });
    connect(closeMonitorSettingItem->button(), &QPushButton::clicked, [=]() {
        m_work->openCloseMonitorSetting();
    });
    connect(lockSaverItem->button(), &QPushButton::clicked, [=]() {
        // 锁屏壁纸/屏幕保护 接口疯狂调用会导致屏幕卡顿，所以设置为2秒调用一次，避免影响控制中心
        lockSaverItem->button()->setEnabled(false);
        QTimer::singleShot(2000, this, [ = ]() {
            lockSaverItem->button()->setEnabled(true);
        });
        // 系统特性，立即打开屏保，会导致按钮无法置灰，延迟500毫秒调用屏保接口
        QTimer::singleShot(500, this, [ = ]() {
            m_work->openLockSaverSetting();
        });
    });
}

ScreenSecurityWidget::~ScreenSecurityWidget()
{
    //注销时，断开相关信号链接
    disconnect(m_work, nullptr, this, nullptr);
}

int ScreenSecurityWidget::sliderValueToLockDelayValue(int sliderValue)
{
    int lockDelay;
    switch(sliderValue) {
    case 1:
        lockDelay = 60;
        break;
    case 2:
        lockDelay = 300;
        break;
    case 3:
        lockDelay = 600;
        break;
    case 4:
        lockDelay = 900;
        break;
    case 5:
        lockDelay = 1800;
        break;
    case 6:
        lockDelay = 3600;
        break;
    case 7:
        lockDelay = 0;
        break;
    default:
        lockDelay = 0;
    }
    return lockDelay;
}

int ScreenSecurityWidget::lockDelayValueToSliderValue(int lockValue)
{
    int sliderValue;
    switch(lockValue) {
    case 60:
        sliderValue = 1;
        break;
    case 300:
        sliderValue = 2;
        break;
    case 600:
        sliderValue = 3;
        break;
    case 900:
        sliderValue = 4;
        break;
    case 1800:
        sliderValue = 5;
        break;
    case 3600:
        sliderValue = 6;
        break;
    case 0:
        sliderValue = 7;
        break;
    default:
        sliderValue = 0;
    }
    return sliderValue;
}

QString ScreenSecurityWidget::sliderValueToLiteralString(const int delay) const
{
    QString strData = "";

    switch (delay) {
    case 1:
        strData = tr("1 Minute");
        break;
    case 2:
        strData = tr("%1 Minutes").arg(5);
        break;
    case 3:
        strData = tr("%1 Minutes").arg(10);
        break;
    case 4:
        strData = tr("%1 Minutes").arg(15);
        break;
    case 5:
        strData = tr("%1 Minutes").arg(30);
        break;
    case 6:
        strData = tr("1 Hour");
        break;
    case 7:
        strData = tr("Never");
        break;
    default:
        strData = tr("%1 Minutes").arg(15);
        break;
    }

    return strData;
}

void ScreenSecurityWidget::changeLinePowerLockDelay(int value)
{
    int lockDelay = sliderValueToLockDelayValue(value);
    m_work->setLinePowerLockDelay(lockDelay);
}

void ScreenSecurityWidget::changeBatteryLockDelay(int value)
{
    int lockDelay = sliderValueToLockDelayValue(value);
    m_work->setBatteryLockDelay(lockDelay);
}
