#include "systemsafetywork.h"

SystemSafetyWork::SystemSafetyWork(SystemSafetyModel *model, QObject *parent)
    : QObject(parent)
    , m_model(model)
    , m_contorl(new controlcenter("com.deepin.dde.ControlCenter", "/com/deepin/dde/ControlCenter", QDBusConnection::sessionBus(), this))
    , m_accounts(new accounts("com.deepin.daemon.Accounts", "/com/deepin/daemon/Accounts", QDBusConnection::systemBus(), this))
    , m_daemonservice(new DaemonService("com.deepin.defender.daemonservice", "/com/deepin/defender/daemonservice",
                                        QDBusConnection::sessionBus(), this))
    , m_powerInter(nullptr)
    , m_desktopInter(nullptr)
    , m_managerInter(nullptr)
    , m_updaterInter(nullptr)
    , m_checkUpdateJob(nullptr)
{
    m_powerInter = new PowerInter("com.deepin.daemon.Power", "/com/deepin/daemon/Power",
                                  QDBusConnection::sessionBus(), this);

    m_desktopInter = new QDBusInterface("com.deepin.dde.desktop",
                                        "/com/deepin/dde/desktop",
                                        "com.deepin.dde.desktop",
                                        QDBusConnection::sessionBus(), this);

    m_managerInter = new ManagerInter("com.deepin.lastore", "/com/deepin/lastore",
                                      QDBusConnection::systemBus(), this);

    m_updaterInter = new UpdaterInter("com.deepin.lastore", "/com/deepin/lastore",
                                      QDBusConnection::systemBus(), this);

    m_systemInfoInter = new SystemInfoInter("com.deepin.daemon.SystemInfo",
                                            "/com/deepin/daemon/SystemInfo",
                                            QDBusConnection::sessionBus(), this);

    //初始化model相关参数
    m_model->setSystemVersionInfo(m_systemInfoInter->version());
    m_model->setIsbAutoCheckUpdates(m_updaterInter->updateNotify());
    m_model->setIsbAutoDownloadUpdates(m_updaterInter->autoDownloadUpdates());

    //创建UsbConnectionWork
    m_usbConnectionWork = new UsbConnectionWork(this);

    //链接来自power dbus的信号
    connect(m_powerInter, &PowerInter::LinePowerLockDelayChanged, this, &SystemSafetyWork::linePowerLockDelayChanged);
    connect(m_powerInter, &PowerInter::BatteryLockDelayChanged, this, &SystemSafetyWork::batteryLockDelayChanged);

    //链接来自 m_updaterInter 的信号
    connect(m_updaterInter, &UpdaterInter::UpdateNotifyChanged, this, [ = ](bool isbAuto) {
        m_model->setIsbAutoCheckUpdates(isbAuto);
        Q_EMIT this->AutoCheckUpdatesChanged(isbAuto);
    });
    connect(m_updaterInter, &UpdaterInter::AutoDownloadUpdatesChanged, this, [ = ](bool isbAuto) {
        m_model->setIsbAutoDownloadUpdates(isbAuto);
        Q_EMIT this->AutoDownloadUpdatesChanged(isbAuto);
    });
    connect(m_daemonservice, &DaemonService::GotoScreenSecurity, this, &SystemSafetyWork::onJumpScreenSecurtiy);
    connect(m_daemonservice, &DaemonService::GotoUpdatePolicy, this, &SystemSafetyWork::onJumpUpdatePolicy);
}

SystemSafetyWork::~SystemSafetyWork()
{

}

bool SystemSafetyWork::haveBattery()
{
    return  m_powerInter->batteryIsPresent().size() > 0;
}

int SystemSafetyWork::getLinePowerLockDelay()
{
    return m_powerInter->linePowerLockDelay();
}

void SystemSafetyWork::setLinePowerLockDelay(int value)
{
    Q_EMIT m_powerInter->setLinePowerLockDelay(value);
}

int SystemSafetyWork::getBatteryLockDelay()
{
    return m_powerInter->batteryLockDelay();
}

void SystemSafetyWork::setBatteryLockDelay(int value)
{
    Q_EMIT m_powerInter->setBatteryLockDelay(value);
}

void SystemSafetyWork::openLockBackgroundSetting()
{
    m_desktopInter->call("ShowWallpaperChooser");
}

void SystemSafetyWork::openLockSaverSetting()
{
    m_desktopInter->call("ShowScreensaverChooser");
}

void SystemSafetyWork::openCloseMonitorSetting()
{
    m_contorl->ShowPage("power", "Plugged In");
}

void SystemSafetyWork::checkForUpdates()
{
    //每次检查更新时，刷新当前版本号
    m_model->setSystemVersionInfo(m_systemInfoInter->version());

    if (checkDbusIsValid()) {
        qDebug() << " checkDbusIsValid . do nothing";
        return;
    }

    m_model->setUpdateStatus(UpdatesStatus::Checking);

    QDBusPendingCall call = m_managerInter->UpdateSource();
    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(call, this);
    connect(watcher, &QDBusPendingCallWatcher::finished, [this, call, watcher] {
        if (!call.isError())
        {
            QDBusReply<QDBusObjectPath> reply = call.reply();
            const QString jobPath = reply.value().path();
            setCheckUpdatesJob(jobPath);
        } else
        {
            m_model->setUpdateStatus(UpdatesStatus::CheckUpdatesFailed);
            if (!m_checkUpdateJob.isNull()) {
                m_managerInter->CleanJob(m_checkUpdateJob->id());
            }
            qWarning() << "UpdateFailed, check for updates error: " << call.error().message();
        }
        watcher->deleteLater();
    });
}

void SystemSafetyWork::setAutoCheckUpdates(bool isbAuto)
{
    m_updaterInter->SetUpdateNotify(isbAuto);
}

void SystemSafetyWork::setAutoDownloadUpdates(bool isbAuto)
{
    m_updaterInter->SetAutoDownloadUpdates(isbAuto);
}

void SystemSafetyWork::openUpdatePage()
{
    m_contorl->ShowHome();
    m_contorl->ShowPage("update", "");
}

bool SystemSafetyWork::checkDbusIsValid()
{
    bool ret = true;

    //如果检查任务已经存在，说明dbus有任务在执行
    if (!m_checkUpdateJob.isNull()) {
        if (m_checkUpdateJob->isValid() && ret) {
            return true;
        } else {
            //执行结束，删除任务
            m_checkUpdateJob->deleteLater();
        }
    }

    return false;
}

void SystemSafetyWork::setCheckUpdatesJob(const QString &jobPath)
{
    if (!m_checkUpdateJob.isNull())
        return;

    m_checkUpdateJob = new JobInter("com.deepin.lastore", jobPath, QDBusConnection::systemBus(), this);
    connect(m_checkUpdateJob, &__Job::StatusChanged, [this](const QString & status) {
        qDebug() << "[setCheckUpdatesJob]status is: " << status;
        if (status == "failed") {
            qWarning() << "check for updates job failed";
            m_managerInter->CleanJob(m_checkUpdateJob->id());

            m_model->setUpdateStatus(UpdatesStatus::CheckUpdatesFailed);

            m_checkUpdateJob->deleteLater();
        } else if (status == "success" || status == "succeed") {
            m_checkUpdateJob->deleteLater();

            //apt update之后可以查看最新更新信息
            QDBusPendingCallWatcher *w = new QDBusPendingCallWatcher(m_updaterInter->ApplicationUpdateInfos(QLocale::system().name()), this);
            connect(w, &QDBusPendingCallWatcher::finished, this, &SystemSafetyWork::onAppUpdateInfoFinished);
        }
    });
}

void SystemSafetyWork::onAppUpdateInfoFinished(QDBusPendingCallWatcher *w)
{
    qDebug() << "onAppUpdateInfoFinished";
    QDBusPendingReply<AppUpdateInfoList> reply = *w;

    if (reply.isError()) {
        const QJsonObject &obj = QJsonDocument::fromJson(reply.
                                                         error().
                                                         message().
                                                         toUtf8())
                                 .object();

        if (obj["Type"].toString().contains("dependenciesBroken")) {
//            m_model->setUpdateStatus(UpdatesStatus::DeependenciesBrokenError);
            m_model->setUpdateStatus(UpdatesStatus::CheckUpdatesFailed);
        } else {
            m_model->setUpdateStatus(UpdatesStatus::CheckUpdatesFailed);
            qWarning() << Q_FUNC_INFO << "UpdateFailed, error msg : " << obj["Type"].toString();
        }

        w->deleteLater();
//        return;
    }

    //检查更新结束
    if (m_updaterInter->updatableApps().size() > 0
            || m_updaterInter->updatablePackages().size() > 0)
        Q_EMIT m_model->sendHaveUpdates(true);
    else {
        Q_EMIT m_model->sendHaveUpdates(false);
    }

    m_model->setUpdateStatus(UpdatesStatus::UpdateDefault);

    w->deleteLater();
}
