#include "usbmanagemonitor.h"
#include "libudev.h"
#include "stdio.h"    // printf(3)
#include "stdlib.h"   // exit(3)
#include "unistd.h"   // fork(3), chdir(3), sysconf(3)
#include "signal.h"   // signal(3)
#include "sys/stat.h" // umask(3)
#include "syslog.h"   // syslog(3), openlog(3), closelog(3)
#include "string.h"   // strncmp
#include "aio.h"      // async I/O
#include "sys/types.h" // aiocb datatype
#include "errno.h"     // EINPROGRESS
#include "fcntl.h"    // open() file descriptor
#include "string.h"   // strsep(3)

#include <QDebug>
#include <QFile>

#define SUBSYSTEM                   "usb"
// 全部允许/允许白名单/全部禁止
#define USB_LIMIT_ALL               "0"
#define USB_LIMIT_PARTY             "1"
#define USB_LIMIT_ZERO              "2"

UsbManageMonitor::UsbManageMonitor(QObject *parent)
    : QObject(parent)
    , m_sFlag("0")
    , m_sWhiteList("")
{
    m_status = 1;
    m_recCount = 0;
    m_disConnectFlag = 1;

}

void UsbManageMonitor::init()
{
    // 首先调用udev_new，创建一个udev library context。
    // udev library context采用引用记数机制，创建的context默认引用记数为1，
    // 使用udev_ref和udev_unref增加或减少引用记数，如果引用记数为0，则释放内部资源
    struct udev *udev = udev_new();
    if (!udev) {
        qDebug() << "Can't create udev.";
    } else {
        qDebug() << "Finished create udev.";
    }

    // 监控设备插拔 udev的设备插拔基于netlink实现
    monitor_devices(udev);
}

void UsbManageMonitor::quit()
{
    m_status = 0;
}

void UsbManageMonitor::setUSBModelAndWhiteList(const QString sFlag, const QString sWhiteList)
{
    // 接收限制模式及白名单
    m_sFlag = sFlag;
    m_sWhiteList = sWhiteList;
}

void UsbManageMonitor::monitor_devices(struct udev *udev)
{
    // 使用udev_monitor_new_from_netlink创建一个新的monitor，
    // 函数的第二个参数是事件源的名称，可选"kernel"或"udev"。
    // 基于"kernel"的事件通知要早于"udev"，但相关的设备结点未必创建完成，
    // 所以一般应用的设计要基于"udev"进行监控
    struct udev_monitor *mon = udev_monitor_new_from_netlink(udev, "udev");

    // 使用udev_monitor_filter_add_match_subsystem_devtype增加一个基于设备类型的udev事件过滤器
    udev_monitor_filter_add_match_subsystem_devtype(mon, SUBSYSTEM, nullptr);
    // 使用udev_monitor_enable_receiving启动监控过程
    udev_monitor_enable_receiving(mon);

    // 监控可以使用udev_monitor_get_fd获取一个文件描述符，基于返回的fd可以执行poll操作，简化程序设计
    int fd = udev_monitor_get_fd(mon);

    while (m_status) {
        fd_set fds;
        FD_ZERO(&fds);
        FD_SET(fd, &fds);

        // Wait for monitor to return
        int ret = select(fd + 1, &fds, nullptr, nullptr, nullptr);
        if (ret <= 0) {
            break;
        }

        if (FD_ISSET(fd, &fds)) {
            //更新参数
//            updateConfig();

            // 插拔事件到达后，可以使用udev_monitor_receive_device获取产生事件的设备映射
            struct udev_device *dev = udev_monitor_receive_device(mon);
            process_device(dev);
        }
    }
}

void UsbManageMonitor::process_device(struct udev_device *dev)
{
//    unsigned long long int seqnum = udev_device_get_seqnum(dev);
//    qDebug() << "seqnum" << seqnum;

//    printf("DeviceNode Path: %s\n", udev_device_get_devnode(dev));
//    printf("Devicesysname: %s\n", udev_device_get_sysname(dev));
//    printf("Devicesyspath: %s\n", udev_device_get_syspath(dev));
//    printf("Devicedevtype: %s\n", udev_device_get_devtype(dev));
//    printf("Device subsystem:%s\n", udev_device_get_subsystem(dev));
//    printf("Device devpath:%s\n", udev_device_get_devpath(dev));
//    printf("Devicesysnum: %s\n", udev_device_get_sysnum(dev));
//    printf("Devicedriver: %s\n", udev_device_get_driver(dev));
//    printf("Deviceaction: %s\n", udev_device_get_action(dev));
//    printf("Device devnum: %ld\n",udev_device_get_devnum(dev));

    const char *product = udev_device_get_sysattr_value(dev, "product");
    const char *serial = udev_device_get_sysattr_value(dev, "serial");
    const char *idVendor = udev_device_get_sysattr_value(dev, "idVendor");
    const char *idProduct = udev_device_get_sysattr_value(dev, "idProduct");
    //设备类 获取值代表 "03"-鼠标 "08"-usb存储设备
    const char *bInterfaceClass = udev_device_get_sysattr_value(dev, "bInterfaceClass");
    const char *bcdUSB = udev_device_get_sysattr_value(dev, "bcdUSB");
    qDebug() << "udev_device_get_sysattr_value product: " << product;
    qDebug() << "udev_device_get_sysattr_value serial: " << serial;
    qDebug() << "udev_device_get_sysattr_value idVendor: " << idVendor;
    qDebug() << "udev_device_get_sysattr_value idProduct: " << idProduct;
    qDebug() << "udev_device_get_sysattr_value bInterfaceClass: " << bInterfaceClass;
    qDebug() << "udev_device_get_sysattr_value bcdUSB: " << bcdUSB;
    qDebug() << "**********************************************";
    QString sUsbIfo = + product +  QString(",") + serial + QString(",") + idVendor + QString(",") + idProduct;

    //使用前两次设备句柄获取设备信息
    if (0 == m_recCount) {
        m_recCount++;
        m_usbDeviceInfo.product = product;
        m_usbDeviceInfo.serial = serial;
        m_usbDeviceInfo.idVendor = idVendor;
        m_usbDeviceInfo.idProduct = idProduct;

        m_usbDeviceInfoStr = sUsbIfo;
        return;
    }

    if (1 == m_recCount) {
        m_recCount++;
        m_usbDeviceInfo.bInterfaceClass = bInterfaceClass;
        return;
    }

    //获取到第4次句柄时不做任何处理，清空临时的设备信息
    if (3 == m_recCount) {
        m_recCount = 0;
        m_usbDeviceInfo = {};
        if (0 == m_disConnectFlag) {
            disconnect_device(dev);
            m_disConnectFlag = 1;
        }
        return;
    }

    //如果不是usb存储设备，则不做限制和记录
    if ("08" != m_usbDeviceInfo.bInterfaceClass){
        m_recCount++;
        return;
    }

    if (USB_LIMIT_ALL == m_sFlag) {
        Q_EMIT checkUsbInsert(true, m_usbDeviceInfoStr);
    } else if (USB_LIMIT_ZERO == m_sFlag) {
        m_disConnectFlag = disconnect_device(dev);
        if (m_disConnectFlag == 0) {
            Q_EMIT checkUsbInsert(false, m_usbDeviceInfoStr);
        } else {
            Q_EMIT checkUsbInsert(true, m_usbDeviceInfoStr);
        }
    } else if (USB_LIMIT_PARTY == m_sFlag) {
        int nFlag = check_whitelist(dev);
        if (nFlag == 0) {
            Q_EMIT checkUsbInsert(true, m_usbDeviceInfoStr);
        } else {
            m_disConnectFlag = disconnect_device(dev);
            if (m_disConnectFlag == 0) {
                Q_EMIT checkUsbInsert(false, m_usbDeviceInfoStr);
            } else {
                Q_EMIT checkUsbInsert(true, m_usbDeviceInfoStr);
            }
        }
    } else {
        Q_EMIT checkUsbInsert(true, m_usbDeviceInfoStr);
    }

    m_recCount++;
}

int UsbManageMonitor::check_whitelist(struct udev_device *dev)
{
    Q_UNUSED(dev);
    // Grab device information
//    const char *serial = udev_device_get_sysattr_value(dev, "serial");
//    const char *vendor = udev_device_get_sysattr_value(dev, "idVendor");
//    const char *product = udev_device_get_sysattr_value(dev, "idProduct");
    const QString serial = m_usbDeviceInfo.serial;
    const QString vendor = m_usbDeviceInfo.idVendor;
    const QString product = m_usbDeviceInfo.idProduct;

    if (serial.isEmpty() || vendor.isEmpty() || product.isEmpty()) {
        return 1;
    }

    QStringList sList = m_sWhiteList.split("\n");
    for (int i = 0; i < sList.count(); ++i) {
        QStringList sListInfo = sList.at(i).split(",");
        if (sListInfo.count() < 3) return 1 ;
        if (serial == sListInfo.at(0) && vendor == sListInfo.at(1) && product == sListInfo.at(2)) {
            //如果设备在白名单中返回false
            return 0;
        }
    }
    return 1;
}

int UsbManageMonitor::disconnect_device(struct udev_device *dev)
{
    // Open file descriptor to the bus unbind interface
    int file = open("/sys/bus/usb/drivers/usb/unbind", O_WRONLY, 0);


    if (file == -1) {
        qDebug() << "open \'unbind\' file failed !";
        return 1;
    }

    // Grab the path to the device
    const char *path = udev_device_get_devpath(dev);

    char *token;
    char *str = strdup(path);
    char *location;


    if (!path) {
        close(file);
        return 1;
    }

    // Grab only the relevant bus location from the path
    while ((token = strsep(&str, "/")) != NULL) {
        location = strdup(token);
    }

    // create the control block structure
    // and buffer to write
    struct aiocb cb;
    char write_buffer[strlen(location) + 1];

    memset(&cb, 0, sizeof(struct aiocb));
    memcpy(write_buffer, location, strlen(location) + 1);
    cb.aio_nbytes = strlen(location) + 1;
    cb.aio_fildes = file;
    cb.aio_offset = 0;
    cb.aio_buf = write_buffer;

    //printf("%lu\n", strlen(write_buffer));
    //printf("%s\n", write_buffer);

    // Async-write this location to the unbind interface
    if (aio_write(&cb) == -1) {
        close(file);
        return 1;
    }

    /* Wait until completion */
    while (aio_error(&cb) == EINPROGRESS);

    // Check for errors
    int err = aio_error(&cb); //错误返回0
    //int ret = aio_return(&cb);

    if (err == 0) {
        close(file);
        return 1;
    }

    close(file);
    return 0;

}

void UsbManageMonitor::updateConfig()
{
    m_sFlag = m_usbMonitorConfig->limitModelStr();
    m_sWhiteList = m_usbMonitorConfig->whiteListStr();
}
